package fr.inra.pappso.csv2ods;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.nio.charset.Charset;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;

/*******************************************************************************
 * Copyright (c) 2012 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 * libOdsStream is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * libOdsStream is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * libOdsStream. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors: Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial
 * API and implementation
 ******************************************************************************/

public class Ods2Csv implements Runnable {
	private static final Logger logger = Logger.getLogger(Ods2Csv.class);

	InputStream source;
	OutputStream sink;
	private char separator = '\t';
	private Charset charset = Charset.availableCharsets().get("UTF-8");

	private NumericFormat numericFormat = NumericFormat.en;

	private File directory = null;

	private boolean isRunning = false;

	public void setSourceInputStream(InputStream fileInputStream)
			throws Exception {
		if (this.isRunning) {
			String message = "unable to setSinkFilename : Ods2Csv is already running";
			logger.error(message);
			throw new Exception(message);
		}
		source = fileInputStream;
	}

	public void setSourceFilename(String inputOdsFilename) throws Exception {
		if (this.isRunning) {
			String message = "unable to setSinkFilename : Ods2Csv is already running";
			logger.error(message);
			throw new Exception(message);
		}
		logger.debug("setSourceFilename " + inputOdsFilename);
		try {
			source = new FileInputStream(inputOdsFilename);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void setSinkDirectoryName(String outputCsvDirectoryname)
			throws Exception {
		if (this.isRunning) {
			String message = "unable to setSinkFilename : Ods2Csv is already running";
			logger.error(message);
			throw new Exception(message);
		}
		logger.debug("setSinkDirectory for multiCsv " + outputCsvDirectoryname);
		this.directory = new File(outputCsvDirectoryname);
	}

	public void setSinkFilename(String outputCsvFilename) throws Exception {

		if (this.isRunning) {
			String message = "unable to setSinkFilename : Ods2Csv is already running";
			logger.error(message);
			throw new Exception(message);
		}

		logger.debug("setSinkFilename " + outputCsvFilename);
		try {
			sink = new FileOutputStream(outputCsvFilename);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void run() {
		this.isRunning = true;
		Ods2CsvDocReaderInterface odsReader = null;
		try {

			if (directory != null) {
				odsReader = new Ods2MultiCsvDocReader(this.source, directory,
						separator, charset);
			} else {
				odsReader = new Ods2CsvDocReader(this.source, sink, separator,
						charset);
			}
			odsReader.setNumericFormat(this.numericFormat);
			odsReader.readOdsSource();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			odsReader.close();
		} catch (IOException e) {
			String message = "IOException closing the doc reader " + e.getMessage();
			logger.error(message);
			// throw new Exception(message);
		}
		this.sink = null;
	}

	public void setSeparator(char optionValue) throws Exception {
		if (this.isRunning) {
			String message = "unable to setSinkFilename : Ods2Csv is already running";
			logger.error(message);
			throw new Exception(message);
		}
		this.separator = optionValue;
	}

	public void setSinkOutputStream(OutputStream out) throws Exception {
		if (this.isRunning) {
			String message = "unable to setSinkFilename : Ods2Csv is already running";
			logger.error(message);
			throw new Exception(message);
		}
		sink = out;
	}

	public void setNumericFormat(NumericFormat numericFormat) throws Exception {
		if (this.isRunning) {
			String message = "unable to setSinkFilename : Ods2Csv is already running";
			logger.error(message);
			throw new Exception(message);
		}
		if (numericFormat == null) {
			return;
		}
		this.numericFormat = numericFormat;
	}

	public OdsDocWriter getOdsDocPipedOutput() throws Exception {
		OdsDocWriter odsTable = null;
		try {

			PipedOutputStream pos = new PipedOutputStream();
			PipedInputStream pis = new PipedInputStream(pos);

			odsTable = new OdsDocWriter(pos);

			this.setSourceInputStream(pis);
			// ods2csv.setSeparator('\t');
			(new Thread(this)).start();
			return odsTable;
			// ods2csv.run();
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new Exception(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new Exception(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new Exception(message);
		}
	}

}
