/*******************************************************************************
 * Copyright (c) 2011 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 *     libOdsStream is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     libOdsStream is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with libOdsStream.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/

package fr.inra.pappso.libodsstream;

import java.io.IOException;
import java.io.InputStream;

import fr.inra.pappso.libodsstream.reader.OdsCell;
import fr.inra.pappso.libodsstream.reader.OdsSheet;
import fr.inra.pappso.libodsstream.saxreader.SaxOdsDocReader;

/**
 * Reads an ODS document from either a file or an input stream. To use it,
 * create first a new class extending OdsDocReader and override needed functions
 * 
 * @author Olivier Langella <Olivier.Langella@moulon.inra.fr>
 * 
 */
abstract public class OdsDocReader extends SaxOdsDocReader {

	/**
	 * creates a ODS file reader
	 * 
	 * @param inputFilename
	 * @throws IOException
	 */
	public OdsDocReader(String inputFilename) throws IOException {
		super(inputFilename);
	}

	/**
	 * creates an ODS reader from an input stream
	 * 
	 * @param in
	 * @throws IOException
	 */
	public OdsDocReader(InputStream in) throws IOException {
		super(in);
	}

	/**
	 * callback that indicates the begining of a data sheet. Override it in
	 * order to retrieve information about the current data sheet.
	 * 
	 * @param sheet
	 * @throws Exception
	 */
	@Override
	public void startSheet(OdsSheet sheet) throws Exception {
	}

	/**
	 * callback that indicates the end of the current data sheet. Override it if
	 * needed
	 * 
	 * @throws Exception
	 */
	@Override
	public void endSheet() throws Exception {
	}

	/**
	 * callback that indicates a new line start. Override it if needed.
	 * 
	 * @throws Exception
	 */
	@Override
	public void startLine() throws Exception {
	}

	/**
	 * callback that indicates a line ending. Override it if needed.
	 * 
	 * @throws Exception
	 */
	@Override
	public void endLine() throws Exception {
	}

	/**
	 * callback that report the content of the current cell in a dedicated Cell
	 * object. Override it if you need to retrieve cell content.
	 * 
	 * @throws Exception
	 */
	@Override
	public void setCell(OdsCell cell) throws Exception {
	}

	/**
	 * callback that report the end of the ODS document. Override it if you need
	 * to know that reading is finished.
	 * 
	 * @throws Exception
	 */
	@Override
	public void endDocument() throws Exception {
	}
}
