/*******************************************************************************
 * Copyright (c) 2011 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 *     libOdsStream is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     libOdsStream is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with libOdsStream.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/
package fr.inra.pappso.libodsstream.reader;

import java.util.GregorianCalendar;

/**
 * represents a cell content, this is produced by the OdsDocReader
 * 
 * @see OdsDocReader
 * @author Olivier Langella <Olivier.Langella@moulon.inra.fr>
 * 
 */
public class OdsCell {

	protected String stringValue = null;
	protected String officeValueType = null;
	protected GregorianCalendar dateValue = null;
	protected Double doubleValue = null;

	protected OdsCell() {
	}

	/**
	 * returns the text content of the cell, if this one contains text, null
	 * otherwise
	 * 
	 * @return the text contained in the cell
	 */
	public String getStringValue() {
		if (officeValueType == null) {
			return null;
		}
		if (officeValueType.endsWith("string")) {
			if (stringValue == null) {
				return "";
			}
			return stringValue;
		}
		return null;
	}

	/**
	 * returns the date content of the cell, if this one contains date, null
	 * otherwise
	 * 
	 * @return
	 */
	public GregorianCalendar getDateValue() {
		return dateValue;
	}

	/**
	 * returns the number content of the cell, if this one contains a number,
	 * null otherwise
	 * 
	 * @return
	 */
	public Double getDoubleValue() {
		return doubleValue;
	}

	public boolean isEmpty() {
		if (officeValueType == null) {
			return true;
		}
		return false;
	}

	/**
	 * tells if this cell contains a string
	 * 
	 * @return boolean
	 */
	public boolean isString() {
		if (isEmpty()) {
			// empty cell
			return false;
		}
		if (officeValueType.endsWith("string")) {
			return true;
		}
		return false;
	}

	/**
	 * tells if this cell contains a date
	 * 
	 * @return boolean
	 */
	public boolean isDate() {
		if (isEmpty()) {
			// empty cell
			return false;
		}
		if (officeValueType.endsWith("date")) {
			return true;
		}
		return false;
	}

	/**
	 * tells if this cell contains a date
	 * 
	 * @return boolean
	 */
	public boolean isNumber() {
		if (isEmpty()) {
			// empty cell
			return false;
		}
		if (isString()) {
			return false;
		}
		if (isDate()) {
			return false;
		}
		return true;
	}

}
