/*******************************************************************************
 * Copyright (c) 2011 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 *     libOdsStream is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     libOdsStream is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with libOdsStream.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/
package fr.inra.pappso.libodsstream.saxreader;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @brief attention ça ne marche pas encore
 * 
 *        base class to build sax parser : define a stack to store tags and use
 *        reflexion to look for functions associated to tag names
 * 
 * @author olivier
 * 
 */

public class SaxBaseHandler extends DefaultHandler {
	protected StringBuffer data;
	protected Stack<String> stack;
	protected Locator xml_location;

	public SaxBaseHandler() {
		super();
		stack = new Stack<String>();
	}

	public void setDocumentLocator(Locator locator) {
		this.xml_location = locator;
	}

	public void startElement(String nameSpaceUri, String sName, String qName,
			Attributes attrs) throws SAXException {
		String tag = sName;
		if ("".equals(tag))
			tag = qName;
		tag = tag.replaceAll("-", "");
		// System.out.println(tag);

		stack.push(tag);
		data = new StringBuffer();
		// String nameSpaceUri, String tag,Attributes attrs
		@SuppressWarnings("rawtypes")
		Class params[] = new Class[] { String.class, Attributes.class };

		try {
			Method m = this.getClass().getDeclaredMethod("startElement" + tag,
					params);

			m.invoke(this, nameSpaceUri, attrs);

		} catch (SecurityException e) {
			e.printStackTrace();
			throw new SAXException("SecurityException for method "
					+ "startElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (NoSuchMethodException e) {
			// System.out.println("method not found " + "startElement" + tag);
			// this function for this tag does not exist, no problem
			try {
				this.startUnknownElement(nameSpaceUri, sName, qName, attrs);
			} catch (SAXNotRecognizedException e1) {
				// e1.printStackTrace();
				throw new SAXNotRecognizedException(
						"this handler does not recognize this file format\n"
								+ e.getMessage());
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			throw new SAXException("illegal argument for method "
					+ "startElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new SAXException("illegal access for method "
					+ "startElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (InvocationTargetException e) {
			String message = "InvocationTargetException for method "
					+ "startElement" + tag + " " + this.getPosition() + "\n"
					+ e.getTargetException().getMessage();
			throw new SAXException(message, (Exception) e.getTargetException());
		}
	}

	protected String getPosition() {
		return " at line " + this.getLineNumber() + " column "
				+ this.getColumnNumber();
	}

	protected String getColumnNumber() {
		return "" + this.xml_location.getColumnNumber();
	}

	protected String getLineNumber() {
		return "" + this.xml_location.getLineNumber();
	}

	/**
	 * @brief launch if the element is unknown to the handler
	 * 
	 * @param nameSpaceUri
	 * @param sName
	 * @param qName
	 * @param attrs
	 * @throws SAXException
	 * @throws ProticPortException
	 */
	protected void startUnknownElement(String nameSpaceUri, String sName,
			String qName, Attributes attrs) throws SAXException {

	}

	public void endElement(String nameSpaceUri, String sName, String qName)
			throws SAXException {
		String tag = sName;
		tag = tag.replaceAll("-", "");
		@SuppressWarnings("rawtypes")
		Class params[] = new Class[] { String.class, String.class };
		if ("".equals(tag))
			tag = qName;
		try {
			Method m = this.getClass().getDeclaredMethod("endElement" + tag,
					params);

			if (data != null) {
				m.invoke(this, nameSpaceUri, data.toString());
			} else {
				m.invoke(this, nameSpaceUri, null);
			}

		} catch (SecurityException e) {
			e.printStackTrace();
			throw new SAXException("SecurityException for method "
					+ "endElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (NoSuchMethodException e) {
			// System.out.println("method not found " + "startElement" + tag);
			// this function for this tag does not exist, no problem
			try {
				this.endUnknownElement(nameSpaceUri, sName, qName,
						data.toString());
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				String message = "Error reading file at end of " + tag
						+ " at line " + this.xml_location.getLineNumber()
						+ "\n" + e1.getMessage();
				throw new SAXException(message, e1);
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			throw new SAXException("illegal argument for method "
					+ "endElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new SAXException("illegal access for method " + "endElement"
					+ tag + " at line " + this.xml_location.getLineNumber());
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			String message = "InvocationTargetException for method "
					+ "endElement" + tag + " " + this.getPosition() + "\n"
					+ e.getTargetException().getMessage();
			throw new SAXException(message, (Exception) e.getTargetException());

		}

		stack.pop();
	}

	protected void endUnknownElement(String nameSpaceUri, String sName,
			String qName, String data) throws Exception {

	}

	/**
	 * this method reads the element content
	 */
	public void characters(char buf[], int offset, int len) throws SAXException {
		data.append(buf, offset, len);
	}

}
