/*******************************************************************************
 * Copyright (c) 2011 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 *     libOdsStream is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     libOdsStream is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with libOdsStream.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/

package fr.inra.pappso.libodsstream.saxreader;

import java.util.GregorianCalendar;

import javax.xml.datatype.DatatypeFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.inra.pappso.libodsstream.reader.OdsCell;
import fr.inra.pappso.libodsstream.reader.OdsSheet;
import fr.inra.pappso.libodsstream.structure.MetaXml;

class SheetSaxed extends OdsSheet {
	protected SheetSaxed(String name) {
		super(name);
	}

	protected void setName(String name) {
		this.name = name;
	}

}

class CellSaxed extends OdsCell {

	protected String getOfficeValueType() {
		return officeValueType;
	}

	protected void setStringValue(String stringValue) {
		this.stringValue = stringValue;
	}

	public void addStringValue(String stringValue) {
		if (stringValue != null) {
			if (this.stringValue == null) {
				this.stringValue = new String();
			}
			this.stringValue += stringValue;
		}
	}

	protected void setOfficeValueType(String value) throws Exception {
		// null value => cell is empty
		this.officeValueType = value;
	}

	protected void setDateValue(GregorianCalendar date) {
		this.dateValue = date;
	}

	protected void setValueDouble(Double value) {
		this.doubleValue = value;
	}

}

public class SaxHandlerContentXml extends SaxBaseHandler {

	private SaxOdsDocReader odsDocReader;

	private CellSaxed currentCell;

	private Integer numberColumnsRepeated;

	private static DatatypeFactory datatypeFactory = null;

	private boolean isInParagraph = false;

	public SaxHandlerContentXml(SaxOdsDocReader saxOdsDocReader)
			throws Exception {
		this.odsDocReader = saxOdsDocReader;

		if (datatypeFactory == null) {
			datatypeFactory = DatatypeFactory.newInstance();
		}
	}

	@Override
	protected String getPosition() {
		return " in sheet " + this.odsDocReader.getSheetPositionStr()
				+ " at line " + this.getLineNumber() + " column "
				+ this.getColumnNumber();
	}

	@Override
	protected String getColumnNumber() {
		return "" + this.odsDocReader.getColumnPositionStr();
	}

	@Override
	protected String getLineNumber() {
		return "" + this.odsDocReader.getLinePosition();
	}

	@Override
	public void endDocument() throws SAXException {
	}

	@Override
	protected void startUnknownElement(String nameSpaceUri, String sName,
			String qName, Attributes attrs) throws SAXException {
		// System.out.println("nameSpaceUri : " + nameSpaceUri);
		// System.out.println("sName : " + sName);
		// System.out.println("qName : " + qName);
	}

	public void startElementtable(String nameSpaceUri, Attributes attrs)
			throws SAXException {
		/*
		 * nameSpaceUri : urn:oasis:names:tc:opendocument:xmlns:table:1.0 sName
		 * : table qName : table:table
		 */
		if (!nameSpaceUri.endsWith(MetaXml.hashNamespaceURI.get("table"))) {
			return;
		}
		try {
			OdsSheet sheet = new SheetSaxed(attrs.getValue("table:name"));
			odsDocReader.setSheetName(sheet.getName());
			this.odsDocReader.startSheet(sheet);
		} catch (Exception e) {
			String message = "ERROR in startElementtable " + e.getMessage();
			throw new SAXException(message);
		}

	}

	protected void endElementtable(String nameSpaceUri, String data)
			throws SAXException {
		if (!nameSpaceUri.equals(MetaXml.hashNamespaceURI.get("table"))) {
			return;
		}
		try {
			this.odsDocReader.endSheet();
		} catch (Exception e) {
			String message = "ERROR in endElementtable " + e.getMessage();
			throw new SAXException(message);
		}

	}

	public void startElementtablerow(String nameSpaceUri, Attributes attrs)
			throws SAXException {
		/*
		 * writer.writeStartElement("table", "table-row",
		 * hashNamespaceURI.get("table"));
		 */
		if (!nameSpaceUri.equals(MetaXml.hashNamespaceURI.get("table"))) {
			return;
		}
		try {
			odsDocReader.incLineNumber();
			this.odsDocReader.startLine();
		} catch (Exception e) {
			String message = "ERROR in startElementtablerow " + e.getMessage();
			throw new SAXException(message);
		}

	}

	protected void endElementtablerow(String nameSpaceUri, String data)
			throws SAXException {
		if (!nameSpaceUri.equals(MetaXml.hashNamespaceURI.get("table"))) {
			return;
		}
		try {
			this.odsDocReader.endLine();
		} catch (Exception e) {
			String message = "ERROR in your endLine function : " + e.getMessage();
			throw new SAXException(message);
		}

	}

	public void startElementtablecell(String nameSpaceUri, Attributes attrs)
			throws SAXException {
		isInParagraph = false;
		/*
		 * writer.writeStartElement("table", "table-row",
		 * hashNamespaceURI.get("table"));
		 */
		if (!nameSpaceUri.equals(MetaXml.hashNamespaceURI.get("table"))) {
			return;
		}
		try {
			this.numberColumnsRepeated = 1;
			currentCell = null;
			String valueTypeStr = attrs.getValue("office:value-type");
			currentCell = new CellSaxed();
			// office:value-type="string"

			currentCell.setOfficeValueType(valueTypeStr);

			// table:number-columns-repeated="2"
			// <table:table-cell table:number-columns-repeated="2"
			// office:value-type="string"><text:p>N.A.</text:p></table:table-cell>
			try {
				if (attrs.getValue("table:number-columns-repeated") != null) {
					this.numberColumnsRepeated = new Integer(
							attrs.getValue("table:number-columns-repeated"));
					// System.out.println("coucou " + numberColumnsRepeated);
				}
			} catch (Exception e) {
				String message = "problem with table:number-columns-repeated  "
						+ e.getMessage();
				throw new Exception(message);
			}

			if (currentCell.getOfficeValueType() != null) {
				try {
					if (attrs.getValue("office:date-value") != null) {
						/*
						 * writer.writeAttribute("office",
						 * hashNamespaceURI.get("office"), "date-value",
						 * cal.toXMLFormat());
						 */
						String dateStr = attrs.getValue("office:date-value");
						if (dateStr == null) {
							throw new Exception("date-value is null");
						}
						GregorianCalendar date = datatypeFactory
								.newXMLGregorianCalendar(dateStr)
								.toGregorianCalendar();
						currentCell.setDateValue(date);
					}
				} catch (Exception e) {
					String message = "problem with date-value  "
							+ e.getMessage();
					throw new Exception(message);
				}

				try {
					if (currentCell.getOfficeValueType().equals("float")) {
						/*
						 * writer.writeAttribute("office",
						 * hashNamespaceURI.get("office"), "value-type",
						 * "float"); writer.writeAttribute("office",
						 * hashNamespaceURI.get("office"), "value", value);
						 */
						String valueStr = attrs.getValue("office:value");
						if (valueStr == null) {
							throw new Exception("office:value is null");
						}
						currentCell.setValueDouble(new Double(attrs
								.getValue("office:value")));
					}
				} catch (Exception e) {
					String message = "problem with float  " + e.getMessage();
					throw new Exception(message);
				}
			}
		} catch (Exception e) {
			String message = "ERROR in startElementtablecell " + e.getMessage();
			throw new SAXException(message);
		}

	}

	public void startElementp(String nameSpaceUri, Attributes attrs)
			throws SAXException {
		isInParagraph = true;
	}

	public void endElementp(String nameSpaceUri, String data)
			throws SAXException {
		/*
		 * writer.writeStartElement("text", "p", hashNamespaceURI.get("text"));
		 */
		if (!nameSpaceUri.equals(MetaXml.hashNamespaceURI.get("text"))) {
			return;
		}
		if (this.currentCell == null) {
			return;
		}
		try {
			// office:value-type="string"

			// currentCell.addStringValue(data);
		} catch (Exception e) {
			String message = "ERROR in startElementtablecell " + e.getMessage();
			throw new SAXException(message);
		}

	}

	protected void endElementtablecell(String nameSpaceUri, String data)
			throws SAXException {
		/*
		 * writer.writeStartElement("table", "table-cell",
		 * hashNamespaceURI.get("table"));
		 */
		if (currentCell == null) {
			String message = "internal odsstream ERROR : cell is null";
			throw new SAXException(message);
		}
		if (!nameSpaceUri.equals(MetaXml.hashNamespaceURI.get("table"))) {
			return;
		}
		if (odsDocReader == null) {
			throw new SAXException("odsDocReader is null");
		}
		try {
			while (this.numberColumnsRepeated > 0) {
				odsDocReader.incColumnNumber();
				odsDocReader.setCell(currentCell);
				this.numberColumnsRepeated--;
			}
		} catch (Exception e) {
			String message = "ERROR in your setCell(OdsCell cell) function ";
			if (currentCell == null) {
				message += " (cell is null)";
			}
			message += ": " + e.getMessage();
			throw new SAXException(message);
		}

	}

	/**
	 * this method reads the element content
	 */
	public void characters(char buf[], int offset, int len) throws SAXException {
		data.append(buf, offset, len);

		if (data != null) {
			if (this.currentCell != null) {
				if (this.isInParagraph) {
					this.currentCell.addStringValue(data.toString());
					this.data = new StringBuffer();
				}
			}
		}
	}

}