package fr.inra.pappso.libodsstream.structure;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.Duration;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import javax.xml.datatype.DatatypeFactory;

public class ContentXml {

	private static final String xsdNamespaceURI = new String(
			"http://www.w3.org/2001/XMLSchema");

	private static final String xsiNamespaceURI = new String(
			"http://www.w3.org/2001/XMLSchema-instance");

	private static DatatypeFactory datatypeFactory = null;

	private static HashMap<String, String> hashNamespaceURI;

	XMLStreamWriter writer = null;

	private boolean tableStarted;

	private boolean tableRowStarted;

	static {
		hashNamespaceURI = MetaXml.hashNamespaceURI;
	}

	public ContentXml(ZipOutputStream zipOutputStream) throws IOException,
			XMLStreamException, DatatypeConfigurationException {

		if (datatypeFactory == null) {
			datatypeFactory = DatatypeFactory.newInstance();
		}

		ZipEntry entry = new ZipEntry("content.xml");
		zipOutputStream.putNextEntry(entry);

		XMLOutputFactory factory = XMLOutputFactory.newInstance();
		OutputStreamWriter noIndentOutput = new OutputStreamWriter(
				zipOutputStream, "UTF8");
		// XMLStreamWriter noIndentWriter =
		// factory.createXMLStreamWriter(noIndentOutput);
		// writer = new IndentingXMLStreamWriter(noIndentWriter);
		writer = factory.createXMLStreamWriter(noIndentOutput);

		this.WriteHeader();
	}

	private void WriteHeader() throws XMLStreamException {
		this.tableRowStarted = false;
		this.tableStarted = false;
		writer.writeStartDocument("UTF-8", "1.0");
		/*
		 * <office:document-content office:version="1.2"
		 * xmlns:anim="urn:oasis:names:tc:opendocument:xmlns:animation:1.0"
		 * xmlns:chart="urn:oasis:names:tc:opendocument:xmlns:chart:1.0"
		 * xmlns:config="urn:oasis:names:tc:opendocument:xmlns:config:1.0"
		 * xmlns:db="urn:oasis:names:tc:opendocument:xmlns:database:1.0"
		 * xmlns:dc="http://purl.org/dc/elements/1.1/"
		 * xmlns:dom="http://www.w3.org/2001/xml-events"
		 * xmlns:dr3d="urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"
		 * xmlns:draw="urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"
		 * xmlns:fo
		 * ="urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"
		 * xmlns:form="urn:oasis:names:tc:opendocument:xmlns:form:1.0"
		 * xmlns:grddl="http://www.w3.org/2003/g/data-view#"
		 * xmlns:math="http://www.w3.org/1998/Math/MathML"
		 * xmlns:meta="urn:oasis:names:tc:opendocument:xmlns:meta:1.0"
		 * xmlns:number="urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"
		 * xmlns:of="urn:oasis:names:tc:opendocument:xmlns:of:1.2"
		 * xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0"
		 * xmlns:ooo="http://openoffice.org/2004/office"
		 * xmlns:oooc="http://openoffice.org/2004/calc"
		 * xmlns:ooow="http://openoffice.org/2004/writer" xmlns:presentation=
		 * "urn:oasis:names:tc:opendocument:xmlns:presentation:1.0"
		 * xmlns:script="urn:oasis:names:tc:opendocument:xmlns:script:1.0"
		 * xmlns:
		 * smil="urn:oasis:names:tc:opendocument:xmlns:smil-compatible:1.0"
		 * xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0"
		 * xmlns:svg="urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"
		 * xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0"
		 * xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"
		 * xmlns:xforms="http://www.w3.org/2002/xforms"
		 * xmlns:xhtml="http://www.w3.org/1999/xhtml"
		 * xmlns:xlink="http://www.w3.org/1999/xlink"
		 * xmlns:xsd="http://www.w3.org/2001/XMLSchema"
		 * xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
		 */
		writer.writeStartElement("office", "document-content",
				hashNamespaceURI.get("office"));
		writer.writeAttribute("office", hashNamespaceURI.get("office"),
				"version", "1.2");
		// writer.setPrefix("manifest", namespaceURI);
		// writer.setDefaultNamespace(namespaceURI);
		// writer.writeStartElement("mzXML");
		Iterator<String> it = hashNamespaceURI.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			writer.writeNamespace(key, hashNamespaceURI.get(key));
		}

		// xmlns:xsd="http://www.w3.org/2001/XMLSchema"
		writer.writeNamespace("xsd", xsdNamespaceURI);

		// xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
		writer.writeNamespace("xsi", xsiNamespaceURI);

		// <office:scripts/>
		writer.writeStartElement("office", "scripts",
				hashNamespaceURI.get("office"));
		writer.writeEndElement();

		// <office:font-face-decls>
		this.writeFontFaceDecls();
		this.writeAutomaticStyles();

		// <office:body>
		writer.writeStartElement("office", "body",
				hashNamespaceURI.get("office"));
		// <office:spreadsheet>
		writer.writeStartElement("office", "spreadsheet",
				hashNamespaceURI.get("office"));

	}

	private void writeFontFaceDecls() throws XMLStreamException {
		writer.writeStartElement("office", "font-face-decls",
				hashNamespaceURI.get("office"));
		/**
		 * <style:font-face style:font-family-generic="swiss"
		 * style:font-pitch="variable" style:name="Arial"
		 * svg:font-family="Arial"/>
		 * */
		writer.writeStartElement("style", "font-face",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"font-family-generic", "swiss");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"font-pitch", "variable");
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "name",
				"Arial");
		writer.writeAttribute("svg", hashNamespaceURI.get("svg"),
				"font-family", "Arial");
		writer.writeEndElement();

		writer.writeEndElement();
	}

	private void writeAutomaticStyles() throws XMLStreamException {

		/*
		 * <office:automatic-styles>
		 * 
		 * 
		 * 
		 * 
		 * 
		 * 
		 * 
		 * 
		 * Writer
		 * 
		 * 
		 * 
		 * </office:automatic-styles>
		 */
		/*
		 * 
		 * <office:automatic-styles>
		 */
		writer.writeStartElement("office", "automatic-styles",
				hashNamespaceURI.get("office"));
		/*
		 * <style:style style:name="ce1" style:family="table-cell"
		 * style:parent-style-name="Default" style:data-style-name="N0" />
		 */
		writer.writeStartElement("style", "style",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "name",
				"ce1");
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "family",
				"table-cell");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"parent-style-name", "Default");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"data-style-name", "N0");
		writer.writeEndElement();

		/*
		 * <style:style style:name="ce2" style:family="table-cell"
		 * style:parent-style-name="Default" style:data-style-name="N36" />
		 */
		writer.writeStartElement("style", "style",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "name",
				"ce2");
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "family",
				"table-cell");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"parent-style-name", "Default");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"data-style-name", "N36");
		writer.writeEndElement();
		// <style:style style:name="co1" style:family="table-column">
		writer.writeStartElement("style", "style",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "name",
				"co1");
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "family",
				"table-column");

		/*
		 * <style:table-column-properties fo:break-before="auto"
		 * style:column-width="2.27541666666667cm" />
		 */
		writer.writeStartElement("style", "table-column-properties",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("fo", hashNamespaceURI.get("fo"), "break-before",
				"auto");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"column-width", "2.27541666666667cm");
		writer.writeEndElement();
		// </style:style>
		writer.writeEndElement();

		// <style:style style:name="co2" style:family="table-column">
		writer.writeStartElement("style", "style",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "name",
				"co2");
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "family",
				"table-column");
		// <style:table-column-properties
		// fo:break-before="auto" style:column-width="2.11666666666667cm" />
		writer.writeStartElement("style", "table-column-properties",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("fo", hashNamespaceURI.get("fo"), "break-before",
				"auto");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"column-width", "2.27541666666667cm");
		writer.writeEndElement();
		// </style:style>
		writer.writeEndElement();
		// <style:style style:name="ro1" style:family="table-row">
		writer.writeStartElement("style", "style",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "name",
				"ro1");
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "family",
				"table-row");

		// <style:table-row-properties
		// style:row-height="15pt" style:use-optimal-row-height="true"
		// fo:break-before="auto" />
		writer.writeStartElement("style", "table-row-properties",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"row-height", "15pt");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"use-optimal-row-height", "true");
		writer.writeAttribute("fo", hashNamespaceURI.get("fo"), "break-before",
				"auto");
		writer.writeEndElement();
		// </style:style>
		writer.writeEndElement();

		// <style:style style:name="ta1" style:family="table"
		// style:master-page-name="mp1">
		writer.writeStartElement("style", "style",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "name",
				"ta1");
		writer.writeAttribute("style", hashNamespaceURI.get("style"), "family",
				"table");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"master-page-name", "mp1");
		// <style:table-properties table:display="true"
		// style:writing-mode="lr-tb" />
		writer.writeStartElement("style", "table-properties",
				hashNamespaceURI.get("style"));
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"display", "true");
		writer.writeAttribute("style", hashNamespaceURI.get("style"),
				"writing-mode", "lr-tb");
		writer.writeEndElement();

		// </style:style>
		writer.writeEndElement();

		writer.writeEndElement();
	}

	synchronized public void close() throws Exception {
		if (writer != null) {
			try {
				writer.writeEndDocument();
				writer.flush();
				writer.close();
				writer = null;
			} catch (Exception e) {
				String message = "error closing the content.xml file inside the ODS structure\n"
						+ e.getMessage();
				throw new Exception(message);
			}
		}

	}

	public void writeSheet(String tableName) throws XMLStreamException {

		if (this.tableRowStarted) {
			writer.writeEndElement();
		}
		if (this.tableStarted) {
			writer.writeEndElement();
		}
		this.tableStarted = true;
		this.tableRowStarted = false;

		/*
		 * <table:table table:name="Feuille1" table:style-name="ta1"
		 * table:print="false">
		 */
		writer.writeStartElement("table", "table",
				hashNamespaceURI.get("table"));
		writer.writeAttribute("table", hashNamespaceURI.get("table"), "name",
				tableName);
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"style-name", "ta1");
		writer.writeAttribute("table", hashNamespaceURI.get("table"), "print",
				"false");

		/*
		 * <table:table-column table:style-name="co1"
		 * table:default-cell-style-name="ce1" />
		 */
		writer.writeStartElement("table", "table-column",
				hashNamespaceURI.get("table"));
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"style-name", "co1");
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"default-cell-style-name", "ce1");
		writer.writeEndElement();

	}

	public void writeLine() throws XMLStreamException {
		if (this.tableStarted == false) {
			this.writeSheet();
		}
		if (this.tableRowStarted) {
			writer.writeStartElement("table", "table-cell",
					hashNamespaceURI.get("table"));
			writer.writeEndElement();
			writer.writeEndElement();
		}
		this.tableRowStarted = true;
		/*
		 * <table:table-row table:style-name="ro1">
		 */
		writer.writeStartElement("table", "table-row",
				hashNamespaceURI.get("table"));
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"style-name", "ro1");

	}

	private void writeSheet() throws XMLStreamException {
		this.writeSheet("default");
	}

	public void writeCell(double value) throws XMLStreamException {
		String representation = Double.toString(value);
		this.writeCellFloat(representation, representation);
	}

	public void writeCell(int value) throws XMLStreamException {
		String representation = Integer.toString(value);
		this.writeCellFloat(representation, representation);
	}

	private void writeCellFloat(String value, String representation)
			throws XMLStreamException {
		if (this.tableRowStarted == false) {
			this.writeLine();
		}
		/*
		 * <table:table-cell office:value-type="float" office:value="45">
		 */
		writer.writeStartElement("table", "table-cell",
				hashNamespaceURI.get("table"));
		writer.writeAttribute("office", hashNamespaceURI.get("office"),
				"value-type", "float");
		writer.writeAttribute("office", hashNamespaceURI.get("office"),
				"value", value);
		// table:style-name="ce1"
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"style-name", "ce1");
		/*
		 * <text:p>45</text:p>
		 */
		writer.writeStartElement("text", "p", hashNamespaceURI.get("text"));
		writer.writeCharacters(representation);
		writer.writeEndElement();

		/*
		 * </table:table-cell>
		 */
		writer.writeEndElement();
		/*
		 * </table:table-row> </table:table> </office:spreadsheet>
		 * </office:body>
		 */
	}

	public void writeCell(String value) throws XMLStreamException {
		if (value == null) {
			value = new String("");
		}
		if (this.tableRowStarted == false) {
			this.writeLine();
		}
		/*
		 * <table:table-cell office:value-type="string">
		 */
		writer.writeStartElement("table", "table-cell",
				hashNamespaceURI.get("table"));
		writer.writeAttribute("office", hashNamespaceURI.get("office"),
				"value-type", "string");
		// table:style-name="ce1"
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"style-name", "ce1");
		/*
		 * <text:p>45</text:p>
		 */
		writer.writeStartElement("text", "p", hashNamespaceURI.get("text"));
		writer.writeCharacters(value.toString());
		writer.writeEndElement();

		writer.writeEndElement();

	}

	public void writeCell(GregorianCalendar cal) throws XMLStreamException,
			DatatypeConfigurationException {
		this.writeCell(datatypeFactory.newXMLGregorianCalendar(cal));
	}

	public void writeCell(XMLGregorianCalendar cal) throws XMLStreamException {
		if (cal == null) {
			this.writeCell("");
		} else {
			if (this.tableRowStarted == false) {
				this.writeLine();
			}
			cal.setTimezone(0);
			/*
			 * <table:table-cell office:value-type="date"
			 * office:date-value="2011-07-16"
			 * ><text:p>16/07/11</text:p></table:table-cell>
			 */
			writer.writeStartElement("table", "table-cell",
					hashNamespaceURI.get("table"));
			writer.writeAttribute("office", hashNamespaceURI.get("office"),
					"value-type", "date");
			writer.writeAttribute("office", hashNamespaceURI.get("office"),
					"date-value", cal.toXMLFormat());
			// table:style-name="ce1"
			writer.writeAttribute("table", hashNamespaceURI.get("table"),
					"style-name", "ce2");
			/*
			 * <text:p>45</text:p>
			 */
			writer.writeStartElement("text", "p", hashNamespaceURI.get("text"));
			// writer.writeCharacters(cal.getDay()+"/"+cal.getMonth()+"/11");
			writer.writeCharacters(cal.getDay() + "/" + cal.getMonth() + "/"
					+ cal.getYear());
			writer.writeEndElement();

			/*
			 * </table:table-cell>
			 */
			writer.writeEndElement();
			/*
			 * </table:table-row> </table:table> </office:spreadsheet>
			 * </office:body>
			 */
		}
	}

	public void writeCell(Date date) throws XMLStreamException,
			DatatypeConfigurationException {
		if (date == null) {
			this.writeCell("");
		} else {
			GregorianCalendar calendar = new GregorianCalendar();
			calendar.setTime(date);
			this.writeCell(calendar);
		}
	}

	public void writeCell(boolean value) throws XMLStreamException {
		if (this.tableRowStarted == false) {
			this.writeLine();
		}
		/*
		 * <table:table-cell office:value-type="date"
		 * office:date-value="2011-07-16"
		 * ><text:p>16/07/11</text:p></table:table-cell>
		 */
		writer.writeStartElement("table", "table-cell",
				hashNamespaceURI.get("table"));
		writer.writeAttribute("office", hashNamespaceURI.get("office"),
				"value-type", "boolean");
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"style-name", "ce1");
		String bvalue = "false";
		if (value) {
			bvalue = "true";
		}
		writer.writeAttribute("office", hashNamespaceURI.get("office"),
				"boolean-value", bvalue);
		writer.writeStartElement("text", "p", bvalue);
		writer.writeEndElement();

		/*
		 * </table:table-cell>
		 */
		writer.writeEndElement();
	}

	public void writeCell(Duration duration) throws XMLStreamException {
		if (duration == null) {
			this.writeCell("");
		} else {
			if (this.tableRowStarted == false) {
				this.writeLine();
			}
			/*
			 * <table:table-cell office:value-type="date"
			 * office:date-value="2011-07-16"
			 * ><text:p>16/07/11</text:p></table:table-cell>
			 */
			writer.writeStartElement("table", "table-cell",
					hashNamespaceURI.get("table"));
			writer.writeAttribute("office", hashNamespaceURI.get("office"),
					"value-type", "time");
			writer.writeAttribute("office", hashNamespaceURI.get("office"),
					"time-value", duration.toString());
			writer.writeAttribute("table", hashNamespaceURI.get("table"),
					"style-name", "ce2");
			/*
			 * <text:p>12:00:00</text:p>
			 */
			writer.writeStartElement("text", "p", hashNamespaceURI.get("text"));
			writer.writeCharacters(duration.getHours() + ":"
					+ duration.getMinutes() + ":" + duration.getSeconds());
			writer.writeEndElement();

			/*
			 * </table:table-cell>
			 */
			writer.writeEndElement();
			/*
			 * </table:table-row> </table:table> </office:spreadsheet>
			 * </office:body>
			 */
		}
	}

	public void writeCell(URI theUri, String description)
			throws XMLStreamException {
		// <table:table-cell office:value-type="string"><text:p><text:a
		// xlink:href="http://linuxfr.org/">linuxfr</text:a></text:p></table:table-cell>
		if (theUri == null) {
			this.writeCell("");
		}
		if (description == null) {
			description = theUri.toString();
		}
		if (this.tableRowStarted == false) {
			this.writeLine();
		}
		/*
		 * <table:table-cell office:value-type="string">
		 */
		writer.writeStartElement("table", "table-cell",
				hashNamespaceURI.get("table"));
		writer.writeAttribute("table", hashNamespaceURI.get("table"),
				"style-name", "ce1");
		writer.writeAttribute("office", hashNamespaceURI.get("office"),
				"value-type", "string");
		writer.writeStartElement("text", "p", hashNamespaceURI.get("text"));
		writer.writeStartElement("text", "a", hashNamespaceURI.get("text"));
		// xlink:href="http://linuxfr.org/"
		writer.writeAttribute("xlink", hashNamespaceURI.get("xlink"), "href",
				theUri.toString());
		writer.writeAttribute("xlink", hashNamespaceURI.get("xlink"), "type",
				"simple");
		writer.writeCharacters(description);
		writer.writeEndElement();
		writer.writeEndElement();

		writer.writeEndElement();

	}

}
