package fr.inra.pappso.libodsstream.structure;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;


public class MetaXml {

	XMLStreamWriter writer;
	// static String namespaceURI = new String
	// ("urn:oasis:names:tc:opendocument:xmlns:manifest:1.0");
	public static HashMap<String, String> hashNamespaceURI;

	static {
		hashNamespaceURI = new HashMap<String, String>();
		hashNamespaceURI.put("anim", new String(
				"urn:oasis:names:tc:opendocument:xmlns:animation:1.0"));
		hashNamespaceURI.put("chart", new String(
				"urn:oasis:names:tc:opendocument:xmlns:chart:1.0"));
		hashNamespaceURI.put("config", new String(
				"urn:oasis:names:tc:opendocument:xmlns:config:1.0"));
		hashNamespaceURI.put("db", new String(
				"urn:oasis:names:tc:opendocument:xmlns:database:1.0"));
		hashNamespaceURI.put("dc", new String(
				"http://purl.org/dc/elements/1.1/"));
		hashNamespaceURI.put("dr3d", new String(
				"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"));
		hashNamespaceURI.put("draw", new String(
				"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"));
		hashNamespaceURI.put("fo", new String(
				"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"));
		hashNamespaceURI.put("form", new String(
				"urn:oasis:names:tc:opendocument:xmlns:form:1.0"));
		hashNamespaceURI.put("grddl", new String(
				"http://www.w3.org/2003/g/data-view#"));
		hashNamespaceURI.put("math", new String(
				"http://www.w3.org/1998/Math/MathML"));
		hashNamespaceURI.put("meta", new String(
				"urn:oasis:names:tc:opendocument:xmlns:meta:1.0"));
		hashNamespaceURI.put("number", new String(
				"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"));
		hashNamespaceURI.put("of", new String(
				"urn:oasis:names:tc:opendocument:xmlns:of:1.2"));
		hashNamespaceURI.put("office", new String(
				"urn:oasis:names:tc:opendocument:xmlns:office:1.0"));
		hashNamespaceURI.put("ooo", new String(
				"http://openoffice.org/2004/office"));
		hashNamespaceURI.put("presentation", new String(
				"urn:oasis:names:tc:opendocument:xmlns:presentation:1.0"));
		hashNamespaceURI.put("script", new String(
				"urn:oasis:names:tc:opendocument:xmlns:script:1.0"));
		hashNamespaceURI.put("smil", new String(
				"urn:oasis:names:tc:opendocument:xmlns:smil-compatible:1.0"));
		hashNamespaceURI.put("style", new String(
				"urn:oasis:names:tc:opendocument:xmlns:style:1.0"));
		hashNamespaceURI.put("svg", new String(
				"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"));
		hashNamespaceURI.put("table", new String(
				"urn:oasis:names:tc:opendocument:xmlns:table:1.0"));
		hashNamespaceURI.put("text", new String(
				"urn:oasis:names:tc:opendocument:xmlns:text:1.0"));
		hashNamespaceURI.put("xforms", new String(
				"http://www.w3.org/2002/xforms"));
		hashNamespaceURI.put("xhtml",
				new String("http://www.w3.org/1999/xhtml"));
		hashNamespaceURI.put("xlink",
				new String("http://www.w3.org/1999/xlink"));
	}

	public MetaXml(ZipOutputStream zipOutputStream) throws IOException,
			XMLStreamException {
		// META-INF/
		/*
		 * manifest.xml <?xml version="1.0" encoding="UTF-8"?>
		 * <manifest:manifest
		 * xmlns:manifest="urn:oasis:names:tc:opendocument:xmlns:manifest:1.0"
		 * manifest:version="1.2" >
		 * 
		 * </manifest:manifest>
		 */
		ZipEntry entry = new ZipEntry("meta.xml");
		zipOutputStream.putNextEntry(entry);

		XMLOutputFactory factory = XMLOutputFactory.newInstance();
		OutputStreamWriter noIndentOutput = new OutputStreamWriter(
				zipOutputStream, "UTF8");
		// XMLStreamWriter noIndentWriter =
		// factory.createXMLStreamWriter(noIndentOutput);

		// writer = new IndentingXMLStreamWriter(noIndentWriter);
		writer = factory.createXMLStreamWriter(noIndentOutput);

		this.WriteHeader();
		/*
		 * <office:meta><meta:generator>ODFDOM/0.8.7</meta:generator>
		 * <meta:creation-date>2011-04-29T16:12:27</meta:creation-date>
		 * <dc:date>2011-04-29T16:12:28</dc:date>
		 * <meta:editing-cycles>1</meta:editing-cycles>
		 * <meta:editing-duration>PT0.602S</meta:editing-duration></office:meta>
		 */
		writer.writeStartElement("office", "meta",
				MetaXml.hashNamespaceURI.get("office"));

		/*
		 * see RFC2616, or 3.1.1 in the oasis doc : meta:generator
		 */
		writer.writeStartElement("meta", "generator",
				MetaXml.hashNamespaceURI.get("meta"));
		writer.writeCharacters("LibOdsStream/0.0.1");
		writer.writeEndElement();

		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		String strdate = dateFormat.format(new Date());

		writer.writeStartElement("meta", "creation-date",
				MetaXml.hashNamespaceURI.get("meta"));
		writer.writeCharacters(strdate);
		writer.writeEndElement();

		writer.writeStartElement("dc", "date",
				MetaXml.hashNamespaceURI.get("dc"));
		writer.writeCharacters(strdate);
		writer.writeEndElement();

		writer.writeStartElement("meta", "editing-cycles",
				MetaXml.hashNamespaceURI.get("meta"));
		writer.writeCharacters("1");
		writer.writeEndElement();

		writer.writeStartElement("meta", "editing-duration",
				MetaXml.hashNamespaceURI.get("meta"));
		writer.writeCharacters("PT0.602S");
		writer.writeEndElement();

		this.close();
	}

	private void WriteHeader() throws XMLStreamException {
		writer.writeStartDocument("UTF-8", "1.0");
		// <manifest:manifest
		// xmlns:manifest="urn:oasis:names:tc:opendocument:xmlns:manifest:1.0"
		// manifest:version="1.2" >
		writer.writeStartElement("office", "document-meta",
				MetaXml.hashNamespaceURI.get("office"));
		writer.writeAttribute("office", MetaXml.hashNamespaceURI.get("office"),
				"version", "1.2");
		// writer.setPrefix("manifest", namespaceURI);
		// writer.setDefaultNamespace(namespaceURI);
		// writer.writeStartElement("mzXML");
		Iterator<String> it = MetaXml.hashNamespaceURI.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			writer.writeNamespace(key, MetaXml.hashNamespaceURI.get(key));

		}

	}

	private void close() throws XMLStreamException {
		writer.writeEndDocument();

		writer.flush();
		writer.close();

	}

}
