/*******************************************************************************
 * Copyright (c) 2011 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 *     libOdsStream is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     libOdsStream is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with libOdsStream.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/
package fr.inra.pappso.libodsstream.structure;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

public class StylesXml {
	XMLStreamWriter writer;
	public static HashMap<String, String> hashNamespaceURI;

	static {
		hashNamespaceURI = new HashMap<String, String>();
		hashNamespaceURI.put("chart", new String(
				"urn:oasis:names:tc:opendocument:xmlns:chart:1.0"));
		hashNamespaceURI.put("dc", new String(
				"http://purl.org/dc/elements/1.1/"));
		hashNamespaceURI.put("dr3d", new String(
				"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"));
		hashNamespaceURI.put("draw", new String(
				"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"));
		hashNamespaceURI.put("fo", new String(
				"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"));
		hashNamespaceURI.put("form", new String(
				"urn:oasis:names:tc:opendocument:xmlns:form:1.0"));
		hashNamespaceURI.put("grddl", new String(
				"http://www.w3.org/2003/g/data-view#"));
		hashNamespaceURI.put("math", new String(
				"http://www.w3.org/1998/Math/MathML"));
		hashNamespaceURI.put("meta", new String(
				"urn:oasis:names:tc:opendocument:xmlns:meta:1.0"));
		hashNamespaceURI.put("number", new String(
				"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"));
		hashNamespaceURI.put("of", new String(
				"urn:oasis:names:tc:opendocument:xmlns:of:1.2"));
		hashNamespaceURI.put("office", new String(
				"urn:oasis:names:tc:opendocument:xmlns:office:1.0"));
		hashNamespaceURI.put("ooo", new String(
				"http://openoffice.org/2004/office"));
		hashNamespaceURI.put("presentation", new String(
				"urn:oasis:names:tc:opendocument:xmlns:presentation:1.0"));
		hashNamespaceURI.put("script", new String(
				"urn:oasis:names:tc:opendocument:xmlns:script:1.0"));
		hashNamespaceURI.put("style", new String(
				"urn:oasis:names:tc:opendocument:xmlns:style:1.0"));
		hashNamespaceURI.put("svg", new String(
				"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"));
		hashNamespaceURI.put("table", new String(
				"urn:oasis:names:tc:opendocument:xmlns:table:1.0"));
		hashNamespaceURI.put("text", new String(
				"urn:oasis:names:tc:opendocument:xmlns:text:1.0"));
		hashNamespaceURI.put("xhtml",
				new String("http://www.w3.org/1999/xhtml"));
		hashNamespaceURI.put("xlink",
				new String("http://www.w3.org/1999/xlink"));
		// * xmlns:ooow="http://openoffice.org/2004/writer"
		hashNamespaceURI.put("ooow", new String(
				"http://openoffice.org/2004/writer"));
		// * xmlns:oooc="http://openoffice.org/2004/calc"
		hashNamespaceURI.put("oooc", new String(
				"http://openoffice.org/2004/calc"));
		// * xmlns:dom="http://www.w3.org/2001/xml-events"
		hashNamespaceURI.put("dom", new String(
				"http://www.w3.org/2001/xml-events"));
		// * xmlns:rpt="http://openoffice.org/2005/report"
		hashNamespaceURI.put("rpt", new String(
				"http://openoffice.org/2005/report"));
		// * xmlns:tableooo="http://openoffice.org/2009/table"
		hashNamespaceURI.put("tableooo", new String(
				"http://openoffice.org/2009/table"));
		// * xmlns:css3t="http://www.w3.org/TR/css3-text/"
		hashNamespaceURI.put("css3t", new String(
				"http://www.w3.org/TR/css3-text/"));
	}

	public StylesXml(ZipOutputStream zipOutputStream) throws Exception {
		ZipEntry entry = new ZipEntry("styles.xml");
		try {
			zipOutputStream.putNextEntry(entry);
		} catch (Exception e) {
			// error adding entry to zip output stream
			throw e;
		}
		XMLOutputFactory factory = XMLOutputFactory.newInstance();
		OutputStreamWriter noIndentOutput = new OutputStreamWriter(
				zipOutputStream, "UTF8");
		// XMLStreamWriter noIndentWriter = factory
		// .createXMLStreamWriter(noIndentOutput);

		// writer = new IndentingXMLStreamWriter(noIndentWriter);
		writer = factory.createXMLStreamWriter(noIndentOutput);

		this.WriteHeader();
		this.WriteFontFaceDecls();

		// <office:styles>
		this.WriteOfficeStyles();

		// <office:automatic-styles>
		this.WriteAutomaticStyles();

		// <office:master-styles>
		this.WriteMasterStyles();

		this.close();
	}

	private void WriteMasterStyles() throws XMLStreamException {
		/*
		 * 
		 * 
		 * <office:master-styles> <style:master-page style:name="mp1"
		 * style:page-layout-name="pm1"> <style:header /> <style:header-left
		 * style:display="false" /> <style:footer /> <style:footer-left
		 * style:display="false" /> </style:master-page> </office:master-styles>
		 */
		writer.writeStartElement("office", "master-styles",
				MetaXml.hashNamespaceURI.get("office"));
		writer.writeStartElement("style", "master-page",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"name", "mp1");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"page-layout-name", "pm1");
		writer.writeStartElement("style", "header",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeEndElement();
		writer.writeStartElement("style", "header-left",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"display", "false");
		writer.writeEndElement();
		writer.writeStartElement("style", "footer-left",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"display", "false");
		writer.writeEndElement();
		writer.writeEndElement();
		writer.writeEndElement();
	}

	private void WriteAutomaticStyles() throws XMLStreamException {
		/*
		 * 
		 * <style:header-style> <style:header-footer-properties
		 * fo:min-height="0.45in" fo:margin-left="0.7in" fo:margin-right="0.7in"
		 * fo:margin-bottom="0in" /> </style:header-style> <style:footer-style>
		 * <style:header-footer-properties fo:min-height="0.45in"
		 * fo:margin-left="0.7in" fo:margin-right="0.7in" fo:margin-top="0in" />
		 * </style:footer-style> </style:page-layout> </office:automatic-styles>
		 */
		/*
		 * <office:automatic-styles> <style:page-layout style:name="pm1">
		 */
		writer.writeStartElement("office", "automatic-styles",
				MetaXml.hashNamespaceURI.get("office"));
		writer.writeStartElement("style", "master-page",
				MetaXml.hashNamespaceURI.get("style"));
		/*
		 * <style:page-layout-properties fo:margin-top="0.3in"
		 * fo:margin-bottom="0.3in" fo:margin-left="0.7in"
		 * fo:margin-right="0.7in" style:table-centering="none"
		 * style:print="objects charts drawings" />
		 */
		writer.writeStartElement("style", "page-layout-properties",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("fo", StylesXml.hashNamespaceURI.get("fo"),
				"margin-top", "0.3in");
		writer.writeAttribute("fo", StylesXml.hashNamespaceURI.get("fo"),
				"margin-bottom", "0.3in");
		writer.writeAttribute("fo", StylesXml.hashNamespaceURI.get("fo"),
				"margin-left", "0.7in");
		writer.writeAttribute("fo", StylesXml.hashNamespaceURI.get("fo"),
				"margin-right", "0.7in");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"table-centering", "none");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"print", "objects charts drawings");
		writer.writeEndElement();

		writer.writeEndElement();
		writer.writeEndElement();

	}

	private void WriteOfficeStyles() throws XMLStreamException {
		// <office:styles>
		writer.writeStartElement("office", "styles",
				MetaXml.hashNamespaceURI.get("office"));

		/*
		 * <number:number-style style:name="N0"><number:number
		 * number:min-integer-digits="1"/></number:number-style>
		 */
		writer.writeStartElement("number", "number-style",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"name", "N0");
		writer.writeStartElement("number", "number",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeAttribute("number",
				StylesXml.hashNamespaceURI.get("number"), "min-integer-digits",
				"1");
		writer.writeEndElement();
		writer.writeEndElement();

		/*
		 * <number:date-style
		 * style:name="N36"><number:day/><number:text>/</number
		 * :text><number:month/><number:text>/</number:text> <number:year/>
		 * </number:date-style>
		 */
		writer.writeStartElement("number", "date-style",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"name", "N36");
		writer.writeStartElement("number", "day",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeEndElement();
		writer.writeStartElement("number", "text",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeCharacters("/");
		writer.writeEndElement();
		writer.writeStartElement("number", "month",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeEndElement();
		writer.writeStartElement("number", "text",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeCharacters("/");
		writer.writeEndElement();
		writer.writeStartElement("number", "year",
				MetaXml.hashNamespaceURI.get("number"));
		writer.writeEndElement();
		writer.writeEndElement();

		/*
		 * 
		 * 
		 * 
		 * 
		 * 
		 * 
		 */
		/*
		 * <style:style style:name="Default" style:family="table-cell"
		 * style:data-style-name="N0">
		 */
		writer.writeStartElement("style", "style",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"name", "Default");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"family", "table-cell");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"data-style-name", "N0");
		/*
		 * <style:table-cell-properties style:vertical-align="automatic"
		 * fo:background-color="transparent" />
		 */
		writer.writeStartElement("style", "table-cell-properties",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"vertical-align", "automatic");
		writer.writeAttribute("fo", StylesXml.hashNamespaceURI.get("fo"),
				"background-color", "transparent");
		writer.writeEndElement();

		/*
		 * <style:text-properties fo:color="#000000" style:font-name="Calibri"
		 * style:font-name-asian="Calibri" style:font-name-complex="Calibri"
		 * fo:font-size="11pt" style:font-size-asian="11pt"
		 * style:font-size-complex="11pt" />
		 */
		writer.writeStartElement("style", "text-properties",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("fo", StylesXml.hashNamespaceURI.get("fo"),
				"color", "#000000");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"font-name", "Arial");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"font-name-asian", "Arial");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"font-name-complex", "Arial");
		writer.writeAttribute("fo", StylesXml.hashNamespaceURI.get("fo"),
				"font-size", "11pt");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"font-size-asian", "11pt");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"font-size-complex", "11pt");
		writer.writeEndElement();

		// </style:style>
		writer.writeEndElement();

		// <style:default-style style:family="graphic">
		writer.writeStartElement("style", "default-style",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"family", "graphic");
		/*
		 * <style:graphic-properties draw:fill="solid" draw:fill-color="#4f81bd"
		 * draw:opacity="100%" draw:stroke="solid" svg:stroke-width="0.02778in"
		 * svg:stroke-color="#385d8a" svg:stroke-opacity="100%" />
		 */
		writer.writeStartElement("style", "graphic-properties",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("draw", StylesXml.hashNamespaceURI.get("draw"),
				"fill", "solid");
		writer.writeAttribute("draw", StylesXml.hashNamespaceURI.get("draw"),
				"fill-color", "#4f81bd");
		writer.writeAttribute("draw", StylesXml.hashNamespaceURI.get("draw"),
				"opacity", "100%");
		writer.writeAttribute("draw", StylesXml.hashNamespaceURI.get("draw"),
				"stroke", "solid");
		writer.writeAttribute("svg", StylesXml.hashNamespaceURI.get("svg"),
				"stroke-width", "0.02778in");
		writer.writeAttribute("svg", StylesXml.hashNamespaceURI.get("svg"),
				"stroke-color", "#385d8a");
		writer.writeAttribute("svg", StylesXml.hashNamespaceURI.get("svg"),
				"stroke-opacity", "100%");
		writer.writeEndElement();

		// </style:default-style>
		writer.writeEndElement();

		writer.writeEndElement();

	}

	private void WriteFontFaceDecls() throws XMLStreamException {
		/*
		 * 
		 * <office:font-face-decls>
		 */
		writer.writeStartElement("office", "font-face-decls",
				MetaXml.hashNamespaceURI.get("office"));
		/*
		 * <style:font-face style:name="Arial" svg:font-family="Arial"
		 * style:font-family-generic="swiss" style:font-pitch="variable"/>
		 */
		// Arial
		writer.writeStartElement("style", "font-face",
				MetaXml.hashNamespaceURI.get("style"));
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"name", "Arial");
		writer.writeAttribute("svg", StylesXml.hashNamespaceURI.get("svg"),
				"font-family", "Arial");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"font-family-generic", "swiss");
		writer.writeAttribute("style", StylesXml.hashNamespaceURI.get("style"),
				"font-pitch", "variable");
		writer.writeEndElement();

		/*
		 * </office:font-face-decls>
		 */
		writer.writeEndElement();
	}

	private void WriteHeader() throws XMLStreamException {
		writer.writeStartDocument("UTF-8", "1.0");
		/*
		 * <?xml version="1.0" encoding="UTF-8"?> <office:document-styles
		 * office:version="1.2" grddl
		 * :transformation="http://docs.oasis-open.org/office/1.2/xslt/odf2rdf.xsl"
		 * >
		 */
		writer.writeStartElement("office", "document-styles",
				StylesXml.hashNamespaceURI.get("office"));
		writer.writeAttribute("office",
				StylesXml.hashNamespaceURI.get("office"), "version", "1.2");
		writer.writeAttribute("grddl", StylesXml.hashNamespaceURI.get("grddl"),
				"transformation",
				"http://docs.oasis-open.org/office/1.2/xslt/odf2rdf.xsl");
		// writer.setPrefix("manifest", namespaceURI);
		// writer.setDefaultNamespace(namespaceURI);
		// writer.writeStartElement("mzXML");
		Iterator<String> it = StylesXml.hashNamespaceURI.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			writer.writeNamespace(key, StylesXml.hashNamespaceURI.get(key));

		}

	}

	private void close() throws XMLStreamException {
		writer.writeEndDocument();

		writer.flush();
		writer.close();

	}

}