/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>,
 * Benoit Valot <Benoit.Valot@moulon.inra.fr> 
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline;

import java.util.Collection;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class Utils {
	private static final Logger logger = Logger.getLogger(Utils.class);

	static private String getPappsoGroupingId(Integer num) {
		int size = (int) java.lang.Math.log10(num);
		size += 97;
		String code = "" + (char) size;
		code += "" + num;
		return (code);
	}

	static public String getPappsoGroupingNumber(Group group, SubGroup sg, int k)
			throws MSMSException {
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (sg == null) {
			String message = "sg is null";
			logger.error(message);
			throw new MSMSException(message);
		}

		return getPappsoGroupingId(group.getNumber()) + "."
				+ getPappsoGroupingId(sg.getNumber()) + "."
				+ getPappsoGroupingId(k);
	}

	static public String getPappsoPeptideMassId(Group group,
			PeptideMass peptideMass) throws MSMSException {
		if (peptideMass == null) {
			String message = "peptideMass is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		return "pep" + getPappsoGroupingId(group.getNumber())
				+ getPappsoGroupingId(peptideMass.getNumber());
	}

	static public String getPappsoGroupId(Group group) throws MSMSException {
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		return getPappsoGroupingId(group.getNumber());
	}

	public static String getPappsoSubGroupId(Group group, SubGroup sg)
			throws MSMSException {
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (sg == null) {
			String message = "sg is null";
			logger.error(message);
			throw new MSMSException(message);
		}

		return getPappsoGroupingId(group.getNumber()) + "."
				+ getPappsoGroupingId(sg.getNumber());
	}

	public static String collectionToString(Collection<?> obsPhosphoPos) {
		String toWrite = "";
		for (Object item : obsPhosphoPos) {
			toWrite += " " + item;
		}
		return toWrite.trim();
	}
}
