/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms;

import java.io.File;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;

public class FastaFile implements Comparable<FastaFile> {
	private static final Logger logger = Logger.getLogger(FastaFile.class);

	private String key;
	private File fastaFile;

	@Override
	public int hashCode() {
		return fastaFile.hashCode();
	}

	@Override
	public boolean equals(Object toCompareIn) {
		FastaFile toCompare = (FastaFile) toCompareIn;
		return (this.fastaFile.equals(toCompare.fastaFile));
	}

	@Override
	public int compareTo(FastaFile fileIn) {
		return (this.fastaFile.compareTo(fileIn.fastaFile));
	}

	public FastaFile(String filePath) {
		this.key = filePath;
		this.fastaFile = new File(filePath);
	}

	public String getKey() {
		return key;
	}

	public String getName() {
		return fastaFile.getName();
	}

	public boolean exists() {
		return fastaFile.exists();
	}

	public String getAbsolutePath() {
		return fastaFile.getAbsolutePath();
	}

	public void checkReadable() throws MSMSException {
		if (!this.fastaFile.exists()) {
			String message = "Fasta file \"" + this.fastaFile.getAbsolutePath()
					+ "\" does not exists";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (!this.fastaFile.canRead()) {
			String message = "Cannot read Fasta file \""
					+ this.fastaFile.getAbsolutePath() + "\"";
			logger.error(message);
			throw new MSMSException(message);
		}
	}

}
