/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein;

import java.util.TreeSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;

public class PeptideMassSet extends TreeSet<PeptideMass> {
	private static final Logger logger = Logger.getLogger(PeptideMassSet.class);

	// private HashMap<PeptideMass, PeptideMass> pm2pm = new
	// HashMap<PeptideMass, PeptideMass>(
	// 0);
	private static final long serialVersionUID = -5367891674312454987L;

	public PeptideMass getInstance(Peptide peptide) throws MSMSException {
		PeptideMass pepMass = new PeptideMass(peptide.get_LI_sequence(),
				peptide.get_mass_Modifs_without_isotope(true).doubleValue());
		if (!this.contains(pepMass)) {
			String message = "no PeptideMass corresonding to this peptide";
			logger.error(message);
			throw new MSMSException(message);
		}
		return this.getInstance(pepMass);
	}

	public PeptideMass addPeptide(Peptide peptide) throws MSMSException {
		PeptideMass pepMass = new PeptideMass(peptide.get_LI_sequence(),
				peptide.get_mass_Modifs_without_isotope(true).doubleValue());

		logger.debug(peptide.get_LI_sequence() + " "
				+ peptide.get_mass_Modifs_without_isotope(true).doubleValue());
		if (this.contains(pepMass)) {
			pepMass = this.getInstance(pepMass);
		} else {
			// pm2pm.put(pepMass, pepMass);
			this.add(pepMass);
		}
		pepMass.addPeptide(peptide);
		return pepMass;
	}

	private PeptideMass getInstance(PeptideMass pepMass) throws MSMSException {
		PeptideMass pepMassRet = this.ceiling(pepMass);
		if (pepMassRet.equals(pepMass)) {
			return pepMassRet;
		} else {
			String message = "pepMassRet (" + pepMassRet.getSequenceLi()
					+ ") != pepMass" + pepMass.getSequenceLi() + " ";
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void numbering() {
		int i = 1;
		for (PeptideMass pepMass : this) {
			pepMass.setNumber(i);
			i++;
			// pm2pm.put(pepMass, pepMass);
		}
	}
}
