package fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile;

import java.io.File;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerXtResultFile4ProcessingParams;
import fr.inra.pappso.xtandempipeline.sax_parse.XtandemPhospho;
import fr.inra.pappso.xtandempipeline.sax_parse.parse_file;
import fr.inra.pappso.xtandempipeline.sax_parse.xtandem;
import fr.inra.pappso.xtandempipeline.sax_parse.xtandem_ms2_spectra;

public class IdentificationXtandemFile extends IdentificationDataFile {
	private static final Logger logger = Logger
			.getLogger(IdentificationXtandemFile.class);

	public IdentificationXtandemFile(File xtandemFile) throws MSMSException {
		super(xtandemFile);
	}

	@Override
	public Identification loadIdentification(
			Identification previousIdentification) throws Exception {
		xtandem tandem = new xtandem(previousIdentification, this);
		new parse_file(tandem, this.file);
		previousIdentification = tandem.get_identifications();
		return previousIdentification;
	}

	@Override
	public Identification loadPhosphoIdentification(
			Identification previousIdentification) throws Exception {
		XtandemPhospho tandem = new XtandemPhospho(previousIdentification, this);
		new parse_file(tandem, this.file);
		previousIdentification = tandem.get_identifications();
		return previousIdentification;
	}

	@Override
	public boolean loadMSMSSpectra(Peptide pep) throws Exception {
		new parse_file(new xtandem_ms2_spectra(pep), this.file);
		return true;
	}

	public void writeProticDbMlDataProcessing(XMLStreamWriter writer)
			throws Exception {
		logger.debug("writeProticDbMlDataProcessing begin");
		HandlerXtResultFile4ProcessingParams handler = new HandlerXtResultFile4ProcessingParams();
		new parse_file(handler, this.file);

		writer.writeStartElement("dataProcessing");
		writer.writeStartElement("software");
		writer.writeStartElement("name");
		writer.writeAttribute("acc", "PROTICdbO:0000283");
		writer.writeCharacters("X!Tandem");
		writer.writeEndElement();// name
		writer.writeStartElement("version");
		writer.writeCharacters(handler.getXtandemVersionNumber());
		writer.writeEndElement();// version
		writer.writeEndElement();// software
		writer.writeStartElement("processingMethod");
		// mzXml source file name
		writeCvParam(writer, "PROTICdbO:0000343",
				handler.getMSMSDataSourceFileName(),
				"MS/MS data source file name");
		// model file name
		writeCvParam(writer, "PROTICdbO:0000342", handler.getXtModelFileName(),
				"X!tandem xml model file name");
		// xtandem result file name
		writeCvParam(writer, "PROTICdbO:0000341", this.file.getName(),
				"X!tandem xml result file name");
		writer.writeEndElement();// processingMethod
		writer.writeEndElement();// dataProcessing

	}

	private void writeCvParam(XMLStreamWriter writer, String acc, String value,
			String description) throws XMLStreamException {
		writer.writeStartElement("cvParam");

		writer.writeAttribute("value", value);
		if (description == null) {
			writer.writeAttribute("name", "N.A.");
		} else {
			writer.writeAttribute("name", description);

		}
		writer.writeAttribute("cvLabel", "PROTICdbO");
		writer.writeAttribute("accession", acc);
		writer.writeEndElement();// cvParam

	}
}
