/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.FastaFile;
import fr.inra.pappso.xtandempipeline.class_msms.FastaFileSet;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;

public class XtandemDatabaseXmlFile {
	private static final Logger logger = Logger
			.getLogger(XtandemDatabaseXmlFile.class);

	private File databaseXmlFile;

	public XtandemDatabaseXmlFile(File tmpDir, FastaFileSet fastaFiles)
			throws MSMSException {
		if (tmpDir == null) {
			try {
				databaseXmlFile = File.createTempFile("database", ".xml");
			} catch (IOException e) {
				String message = "temporary file creation failed for in newDatabaseXmlFile (databases xml file for xtandem)";
				logger.error(message);
				throw new MSMSException(message);
			}
		} else {
			databaseXmlFile = new File(tmpDir.getAbsolutePath()
					+ System.getProperty("file.separator") + "database.xml");
		}
		msms_output out = new msms_output();
		out.open(databaseXmlFile.getAbsolutePath());
		try {
			out.write("<?xml version=\"1.0\"?>" + "\n");
			out.write("<bioml label=\"x! taxon-to-file matching list\">" + "\n");
			out.write("<taxon label=\"usedefined\">" + "\n");
			for (FastaFile fastaFile : fastaFiles) {
				out.write("<file format=\"peptide\" URL=\""
						+ fastaFile.getAbsolutePath() + "\" />" + "\n");
				fastaFile.checkReadable();
			}
			out.write("</taxon>" + "\n");
			out.write("</bioml>" + "\n");
		} finally {
			out.close();
		}
	}

	public String getName() {
		return databaseXmlFile.getName();
	}

	public boolean exists() {
		return databaseXmlFile.exists();
	}

	public String getAbsolutePath() {
		return databaseXmlFile.getAbsolutePath();
	}

	public boolean delete() {
		return databaseXmlFile.delete();
	}

}
