/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.PeakListFile;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;

public class XtandemParamXmlFile {
	private static final Logger logger = Logger
			.getLogger(XtandemParamXmlFile.class);

	private File paramFile;

	public XtandemParamXmlFile(File tmpDir, XtandemModelFile modelXmlFile,
			XtandemDatabaseXmlFile databasexml, PeakListFile pklFile,
			XtandemResultFile result_out) throws MSMSException {
		try {
			if (tmpDir == null) {
				paramFile = File.createTempFile("xtandem_param", ".xml");
			} else {
				paramFile = File
						.createTempFile("xtandem_param", ".xml", tmpDir);
			}
		} catch (IOException e) {
			String message = "temporary file creation failed for in newXtandemParamXmlFile (parameter xml file for xtandem)";
			logger.error(message);
			throw new MSMSException(message);
		}

		// parsing
		// creation du fichier infile
		msms_output out = new msms_output();
		out.open(paramFile.getAbsolutePath());
		out.write("<?xml version=\"1.0\"?>" + "\n");
		out.write("<bioml label=\"" + result_out.getName() + "\">" + "\n");
		out.write("<note type=\"heading\">Paths</note>" + "\n");
		out.write("<note type=\"input\" label=\"list path, default parameters\">"
				+ modelXmlFile.getAbsolutePath() + "</note>" + "\n");
		out.write("<note type=\"input\" label=\"list path, taxonomy information\">"
				+ databasexml.getAbsolutePath() + "</note>" + "\n");
		out.write("<note type=\"input\" label=\"spectrum, path\">"
				+ pklFile.getAbsolutePath() + "</note>" + "\n");
		out.write("<note type=\"heading\">Protein general</note>" + "\n");
		out.write("<note type=\"input\" label=\"protein, taxon\">usedefined</note>"
				+ "\n");
		out.write("<note type=\"heading\">Output</note>" + "\n");
		out.write("<note type=\"input\" label=\"output, path\">"
				+ result_out.getAbsolutePath() + "</note>" + "\n");
		out.write("</bioml>" + "\n");
		out.close();
	}

	public String getName() {
		return paramFile.getName();
	}

	public boolean exists() {
		return paramFile.exists();
	}

	public String getAbsolutePath() {
		return paramFile.getAbsolutePath();
	}

	public boolean delete() {
		return paramFile.delete();
	}

}
