/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.TreeSet;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMassSet;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;

public class OdsPeptideOutput {
	private static final Logger logger = Logger
			.getLogger(OdsPeptideOutput.class);

	protected OdsDocWriter odsTable;

	protected OdsPeptideOutput() {

	}

	public OdsPeptideOutput(OutputStream outputStream) throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			odsTable.writeSheet("peptides");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsPeptideOutput(OdsDocWriter odsTable) throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("peptides");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsPeptideOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Group ID Peptide ID Sequence Modifs Charge MH+ theo Number of
			// subgroup subgroups ids number of spectra

			odsTable.writeLine();
			odsTable.writeCell("Group ID");
			odsTable.writeCell("Peptide ID");
			odsTable.writeCell("Sequence");
			odsTable.writeCell("Modifs");
			odsTable.writeCell("Charge");
			odsTable.writeCell("MH+ theo");
			odsTable.writeCell("Number of subgroups");
			odsTable.writeCell("Subgroup ids");
			odsTable.writeCell("Number of spectra");
			// odsTable.writeCell("Delta-ppm");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	public void write(Identification ident) throws MSMSException {
		try {

			this.writeHeaders(ident);
			for (Group group : ident.getGrouping().getGroupList()) {
				PeptideMassSet pepMassSet = new PeptideMassSet();
				int nbpeptide = 0;
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {

					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						for (Peptide peptide : ((Match) hashProt
								.getGrpProtMatch())
								.get_peptide_order_inclus_redon()) {
							// first pass to index each ScanSample and give it a
							// PeptideMass
							nbpeptide++;
							PeptideMass pepMass = pepMassSet
									.addPeptide(peptide);
							pepMass.addSubGroup(sg);
							// HashPeptide p = peptide.getHashPeptide();
							// if
							// (uniquePeptideSet.add(peptide.getHashPeptide()))
							// {
							// this.writeOnePeptide(group, sg, prot, peptide);
							// }

						}
					}
				}
				logger.debug("nb peptide : " + nbpeptide);

				logger.debug("pepMassSet size : " + pepMassSet.size());

				logger.debug("pepMassSet numbering");
				pepMassSet.numbering();

				for (PeptideMass pepMass : pepMassSet) {
					this.writeOnePeptideMass(ident, group, pepMass);
				}

			}
			odsTable.writeLine();
			odsTable.writeLine();

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOnePeptideMass(Identification ident, Group group,
			PeptideMass pepMass) throws MSMSException {
		try {
			logger.debug("writeOnePeptideMass begin " + group + pepMass);

			odsTable.writeLine();
			// odsTable.writeCell("Group ID");
			odsTable.writeCell(Utils.getPappsoGroupId(group));

			// odsTable.writeCell("Peptide ID");
			odsTable.writeCell(Utils.getPappsoPeptideMassId(group, pepMass));

			// odsTable.writeCell("Sequence LI");
			odsTable.writeCell(pepMass.getSequenceLi());

			Peptide peptide = pepMass.getBestEvaluePeptide();
			// odsTable.writeCell("Modifs");
			odsTable.writeCell(peptide.get_Modifs_formater());
			// odsTable.writeCell("Charge");
			odsTable.writeCell(peptide.get_charge());

			// odsTable.writeCell("MH+ theo");
			odsTable.writeCell(peptide.get_mhplus_theo());
			SubGroupSet sgList = pepMass.getSubGroupSet();
			// odsTable.writeCell("Number of subgroups");
			odsTable.writeCell(sgList.size());
			// odsTable.writeCell("Subgroup ids");

			Collection<String> sgIds = new TreeSet<String>();
			for (SubGroup sgOther : sgList) {
				sgIds.add(Utils.getPappsoSubGroupId(group, sgOther));
			}
			String tagSg = "";
			for (String sgId : sgIds) {
				tagSg += sgId + " ";
			}
			tagSg = tagSg.trim();
			odsTable.writeCell(tagSg);

			// odsTable.writeCell("Number of spectra");
			odsTable.writeCell(pepMass.getHashSampleScanSet().size());

			// S.append("\t" + pep.get_mhplus_theo());
			// odsTable.writeCell(peptide.get_deltamass());

			// nbSpectra = pepMass.getNumberOfPsm();
			// S.append("\t" + pep.get_deltamass());
			// odsTable.writeCell(((peptide.get_deltamass() / peptide
			// .get_mhplus_theo()) * 1000000));

			logger.debug("writeOnePeptideMass end");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeOnePeptideMass "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeOnePeptideMass "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
