/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.TreeSet;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMassSet;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPhosphoModif;

public class OdsPhosphoSpectraOutput {
	private static final Logger logger = Logger
			.getLogger(OdsPhosphoSpectraOutput.class);

	protected OdsDocWriter odsTable;

	protected OdsPhosphoSpectraOutput() {

	}

	public OdsPhosphoSpectraOutput(OutputStream outputStream)
			throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			odsTable.writeSheet("spectra");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsPhosphoSpectraOutput(OdsDocWriter odsTable) throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("spectra");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsPhosphoSpectraOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Group Peptide ID Sample Scan Rt Sequence (top) Modifs (Top) Best
			// Position in peptide Phosphopeptide All Position in sequence
			// Sub-groups Ids Best E-value Charge MH+ Obs MH+ theo DeltaMH+
			// Delta-ppm

			odsTable.writeLine();
			odsTable.writeCell("Group ID");
			odsTable.writeCell("Peptide ID");
			odsTable.writeCell("Sample");
			odsTable.writeCell("Scan");
			odsTable.writeCell("Rt");
			odsTable.writeCell("Sequence (top)");
			odsTable.writeCell("Modifs (top)");
			odsTable.writeCell("Best position in peptide");
			odsTable.writeCell("Phosphopeptide");
			odsTable.writeCell("All Position in sequence");
			odsTable.writeCell("Number of subgroups");
			odsTable.writeCell("Sub-groups Ids");
			odsTable.writeCell("E-value");
			odsTable.writeCell("Charge");
			odsTable.writeCell("MH+ Obs");
			odsTable.writeCell("MH+ theo");
			odsTable.writeCell("DeltaMH+");
			odsTable.writeCell("Delta-ppm");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	public void write(Identification ident) throws MSMSException {
		try {

			this.writeHeaders(ident);
			for (Group group : ident.getGrouping().getGroupList()) {
				PeptideMassSet pepMassSet = new PeptideMassSet();
				HashMap<PeptideMass, HashSet<Peptide>> hashPeptideMass2Peptides = new HashMap<PeptideMass, HashSet<Peptide>>();
				int nbpeptide = 0;
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {

					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						for (Peptide peptide : ((Match) hashProt
								.getGrpProtMatch())
								.get_peptide_order_inclus_redon()) {
							// first pass to index each ScanSample and give it a
							// PeptideMass
							nbpeptide++;
							PeptideMass pepMass = pepMassSet
									.addPeptide(peptide);
							pepMass.addSubGroup(sg);

							HashSet<Peptide> peptideList;
							if (hashPeptideMass2Peptides.containsKey(pepMass)) {
								peptideList = hashPeptideMass2Peptides
										.get(pepMass);
							} else {
								peptideList = new HashSet<Peptide>();
								hashPeptideMass2Peptides.put(pepMass,
										peptideList);
							}
							peptideList.add(peptide);

						}
					}
				}
				logger.debug("nb peptide : " + nbpeptide);

				logger.debug("pepMassSet size : " + pepMassSet.size());

				logger.debug("pepMassSet numbering");
				pepMassSet.numbering();

				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					HashSet<HashSampleScan> uniqueHashSampleScan = new HashSet<HashSampleScan>();

					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						Protein prot = (Protein) hashProt.getGrpProtMatch()
								.getGrpProtein();
						HashSet<PeptideMass> uniquePepMass = new HashSet<PeptideMass>();

						for (Peptide peptide : ((Match) hashProt
								.getGrpProtMatch())
								.get_peptide_order_inclus_redon()) {

							PeptideMass pepMass = pepMassSet
									.getInstance(peptide);
							// pepMass.addSubGroup(sg);
							// HashPeptide p = peptide.getHashPeptide();
							if (uniquePepMass.add(pepMass)) {

								for (HashSampleScan scan : pepMass
										.getHashSampleScanSet()) {

									if (uniqueHashSampleScan.add(scan)) {
										this.writeOneHashSampleScan(ident,
												group, sg, prot, pepMass, scan,
												hashPeptideMass2Peptides
														.get(pepMass));
									}
								}
							}

						}
					}
				}

			}
			odsTable.writeLine();
			odsTable.writeLine();

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOneHashSampleScan(Identification ident, Group group,
			SubGroup sg, Protein prot, PeptideMass pepMass,
			HashSampleScan scan, HashSet<Peptide> peptideSet)
			throws MSMSException {
		try {
			logger.debug("writeOneHashSampleScan begin " + group + pepMass);
			HashSet<Peptide> peptideSetScan = new HashSet<Peptide>();
			MsRun msRun = (MsRun) scan.getGrpMsSample();
			Peptide peptideScan = null;
			for (Peptide peptide : peptideSet) {
				if (peptide.getMsRun().equals(msRun)) {
					peptideSetScan.add(peptide);
					if (peptide.getHashSampleScan() == scan) {
						peptideScan = peptide;
					}
				}
			}
			if (peptideScan == null) {
				// throw new MSMSException("peptideScan == null");
				return;
			}
			odsTable.writeLine();
			// Peptide bestPeptide = pepMass.getBestEvaluePeptide();

			// odsTable.writeCell("Group ID");
			odsTable.writeCell(Utils.getPappsoGroupId(group));
			// odsTable.writeCell("Peptide ID");
			odsTable.writeCell(Utils.getPappsoPeptideMassId(group, pepMass));
			// odsTable.writeCell("Sample");

			odsTable.writeCell(msRun.getSampleName());
			// odsTable.writeCell("Scan");
			odsTable.writeCell(scan.getScan());
			// odsTable.writeCell("Rt");
			odsTable.writeCell(peptideScan.get_RT());
			// odsTable.writeCell("Sequence (top)");
			odsTable.writeCell(peptideScan.get_LI_sequence());
			// odsTable.writeCell("Modifs (top)");
			odsTable.writeCell(peptideScan.get_Modifs_formater());

			Collection<GrpPhosphoModif> phosphoModifs = peptideScan
					.getGrpPhosphoModifs();
			TreeSet<Integer> obsPhosphoPos = new TreeSet<Integer>();
			for (GrpPhosphoModif phosphoModif : phosphoModifs) {
				Modifs modif = (Modifs) phosphoModif;
				obsPhosphoPos.add(modif.get_posi());
			}
			// odsTable.writeCell("Best Position in sequence");
			odsTable.writeCell(Utils.collectionToString(obsPhosphoPos));

			odsTable.writeCell("Phosphopeptide");
			// odsTable.writeCell(pepMass.getSequenceLi());

			HashSet<GrpPhosphoModif> phosphoAllModifs = new HashSet<GrpPhosphoModif>();
			for (Peptide peptide : peptideSet) {
				phosphoAllModifs.addAll(peptide.getGrpPhosphoModifs());
			}
			obsPhosphoPos = new TreeSet<Integer>();
			for (GrpPhosphoModif phosphoModif : phosphoModifs) {
				Modifs modif = (Modifs) phosphoModif;
				obsPhosphoPos.add(modif.get_posi());
			}
			// odsTable.writeCell("All Positions in sequence");
			odsTable.writeCell(Utils.collectionToString(obsPhosphoPos));

			SubGroupSet sgList = group.getSubGroupContaining(scan);
			Collection<String> sgIds = new TreeSet<String>();
			for (SubGroup sgOther : sgList) {
				sgIds.add(Utils.getPappsoSubGroupId(group, sgOther));
			}
			// odsTable.writeCell("Number of subgroups");
			odsTable.writeCell(sgList.size());

			// odsTable.writeCell("Sub-groups Ids");
			odsTable.writeCell(Utils.collectionToString(sgIds));

			// odsTable.writeCell("Best E-value");
			odsTable.writeCell(peptideScan.get_evalue());
			// odsTable.writeCell("Charge");
			odsTable.writeCell(peptideScan.get_charge());
			// odsTable.writeCell("MH+ Obs");
			odsTable.writeCell(peptideScan.get_mhplus_obser());
			// odsTable.writeCell("MH+ theo");
			odsTable.writeCell(peptideScan.get_mhplus_theo());
			// odsTable.writeCell("DeltaMH+");
			odsTable.writeCell(peptideScan.get_deltamass());
			// odsTable.writeCell("Delta-ppm");
			odsTable.writeCell(((peptideScan.get_deltamass() / peptideScan
					.get_mhplus_theo()) * 1000000));

			logger.debug("writeOneHashSampleScan end");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeOneHashSampleScan "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeOneHashSampleScan "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
