package fr.inra.pappso.xtandempipeline.sax_parse;

/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>,
 * Benoit Valot <Benoit.Valot@moulon.inra.fr> 
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 ******************************************************************************/

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Stack;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.helpers.DefaultHandler;

public class SaxBaseHandler extends DefaultHandler {
	private static final Logger logger = Logger.getLogger(SaxBaseHandler.class);

	protected StringBuffer data;
	protected Stack<String> stack;
	protected Locator xml_location;

	public SaxBaseHandler() {
		super();
		stack = new Stack<String>();
	}

	public boolean checkPreviousElementName(String elementName, int distance) {
		int position = stack.size() - distance - 1;
		if (position < 0) {
			return false;
		}
		if (position >= stack.size()) {
			return false;
		}
		if (this.stack.get(stack.size() - distance - 1).equals(elementName)) {
			return true;
		}
		return false;
	}

	public void setDocumentLocator(Locator locator) {
		this.xml_location = locator;
	}

	public void startElement(String nameSpaceUri, String sName, String qName,
			Attributes attrs) throws SAXException {
		String tag = sName;
		if ("".equals(tag))
			tag = qName;

		stack.push(tag);
		data = new StringBuffer();
		// String nameSpaceUri, String tag,Attributes attrs
		@SuppressWarnings("rawtypes")
		Class params[] = new Class[] { String.class, Attributes.class };

		try {
			Method m = this.getClass().getDeclaredMethod("startElement" + tag,
					params);

			m.invoke(this, nameSpaceUri, attrs);

		} catch (SecurityException e) {
			e.printStackTrace();
			throw new SAXException("SecurityException for method "
					+ "startElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (NoSuchMethodException e) {
			// System.out.println("method not found " + "startElement" + tag);
			// this function for this tag does not exist, no problem
			try {
				this.startUnknownElement(nameSpaceUri, sName, qName, attrs);
			} catch (SAXNotRecognizedException e1) {
				// e1.printStackTrace();
				throw new SAXNotRecognizedException(
						"this handler does not recognize this file format\n"
								+ e.getMessage());
			} catch (Exception e1) {
				logger.error("ERROR in startUnknownElement function, tag "
						+ tag + " at line " + this.xml_location.getLineNumber()
						+ " and column " + this.xml_location.getColumnNumber()
						+ "\n" + e1.getMessage());
				throw new SAXException("ERROR opening " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber(), e1);
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			throw new SAXException("illegal argument for method "
					+ "startElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new SAXException("illegal access for method "
					+ "startElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (InvocationTargetException ex) {
			logger.error("ERROR in startElement" + tag + " function, tag "
					+ tag + " at line " + this.xml_location.getLineNumber()
					+ " and column " + this.xml_location.getColumnNumber()
					+ "\n" + ex.getMessage());
			Throwable cause = ex.getCause();
			if (cause == null) {
				throw new SAXException("ERROR opening " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber()
						+ "\n cause is null" + ex.getMessage());
			} else if (cause instanceof RuntimeException) {
				throw new SAXException("ERROR opening " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber()
						+ " RuntimeException :\n",
						(RuntimeException) ex.getCause());
			} else if (cause instanceof Exception) {
				logger.error(((Exception) ex.getCause()).getMessage());
				throw new SAXException("ERROR opening " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber(),
						(Exception) ex.getCause());
			} else {
				throw new SAXException("ERROR opening " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber() + "\n"
						+ ex.getMessage());
			}

		} catch (Exception e1) {
			logger.error("ERROR in startElement" + tag + " function, tag "
					+ tag + " at line " + this.xml_location.getLineNumber()
					+ " and column " + this.xml_location.getColumnNumber()
					+ "\n" + e1.getMessage());
			throw new SAXException("ERROR opening " + tag + " element at line "
					+ this.xml_location.getLineNumber() + " and column "
					+ this.xml_location.getColumnNumber(), e1);
		}
	}

	/**
	 * @brief launch if the element is unknown to the handler
	 * 
	 * @param nameSpaceUri
	 * @param sName
	 * @param qName
	 * @param attrs
	 * @throws SAXException
	 * @throws ProticPortException
	 */
	protected void startUnknownElement(String nameSpaceUri, String sName,
			String qName, Attributes attrs) throws Exception {

	}

	public void endElement(String nameSpaceUri, String sName, String qName)
			throws SAXException {
		String tag = sName;
		@SuppressWarnings("rawtypes")
		Class params[] = new Class[] { String.class, String.class };
		if ("".equals(tag))
			tag = qName;
		try {
			Method m = this.getClass().getDeclaredMethod("endElement" + tag,
					params);

			if (data != null) {
				m.invoke(this, nameSpaceUri, data.toString());
			} else {
				m.invoke(this, nameSpaceUri, null);
			}

		} catch (SecurityException e) {
			e.printStackTrace();
			throw new SAXException("SecurityException for method "
					+ "endElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (NoSuchMethodException e) {
			// System.out.println("method not found " + "startElement" + tag);
			// this function for this tag does not exist, no problem
			try {
				this.endUnknownElement(nameSpaceUri, sName, qName,
						data.toString());
			} catch (Exception e1) {
				logger.error("ERROR in endUnknownElement function, tag " + tag
						+ " at line " + this.xml_location.getLineNumber()
						+ " and column " + this.xml_location.getColumnNumber()
						+ "\n" + e1.getMessage());
				throw new SAXException("ERROR closing " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber(), e1);
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			throw new SAXException("illegal argument for method "
					+ "endElement" + tag + " at line "
					+ this.xml_location.getLineNumber());
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new SAXException("illegal access for method " + "endElement"
					+ tag + " at line " + this.xml_location.getLineNumber());
		} catch (InvocationTargetException ex) {
			logger.error("ERROR in endElement" + tag + " function, tag " + tag
					+ " at line " + this.xml_location.getLineNumber()
					+ " and column " + this.xml_location.getColumnNumber()
					+ "\n" + ex.getMessage());
			Throwable cause = ex.getCause();
			if (cause == null) {
				throw new SAXException("ERROR closing " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber()
						+ "\n cause is null" + ex.getMessage());
			} else if (cause instanceof RuntimeException) {
				throw new SAXException("ERROR closing " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber()
						+ " RuntimeException :\n",
						(RuntimeException) ex.getCause());
			} else if (cause instanceof Exception) {
				logger.error(((Exception) ex.getCause()).getMessage());
				throw new SAXException("ERROR closing " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber(),
						(Exception) ex.getCause());
			} else {
				throw new SAXException("ERROR closing " + tag
						+ " element at line "
						+ this.xml_location.getLineNumber() + " and column "
						+ this.xml_location.getColumnNumber() + "\n"
						+ ex.getMessage());
			}

		} catch (Exception e1) {
			logger.error("ERROR in endElement" + tag + " function, tag " + tag
					+ " at line " + this.xml_location.getLineNumber()
					+ " and column " + this.xml_location.getColumnNumber()
					+ "\n" + e1.getMessage());
			throw new SAXException("ERROR closing " + tag + " element at line "
					+ this.xml_location.getLineNumber() + " and column "
					+ this.xml_location.getColumnNumber(), e1);
		}

		stack.pop();
	}

	protected void endUnknownElement(String nameSpaceUri, String sName,
			String qName, String data) throws Exception {

	}

	/**
	 * this method reads the element content
	 */
	public void characters(char buf[], int offset, int len) throws SAXException {
		data.append(buf, offset, len);
	}

}
