package fr.inra.pappso.xtandempipeline.thread;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashSet;

import fr.inra.pappso.csv2ods.Ods2Csv;
import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.ModifsList;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_peptide_fdr_distribution;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_protein_fdr_distribution;
import fr.inra.pappso.xtandempipeline.filter_print.filter_isotopic_modifs;
import fr.inra.pappso.xtandempipeline.filter_print.print_fasta;
import fr.inra.pappso.xtandempipeline.filter_print.print_pepnovo;
import fr.inra.pappso.xtandempipeline.output.MassChroQmlOutput;
import fr.inra.pappso.xtandempipeline.output.ProticDbMlOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPeptideOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPeptidePosOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsProteinOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsSpectraOutput;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;

public class printing_base extends process {

	private File out;

	protected ArrayList<Identification> identifications = XtandemPipelineSession
			.getInstance().getIdentificationList();

	private print_fasta fasta_type;

	private ModifsList isotopic_mods;

	protected boolean isCsv;

	protected boolean isExpertMode = false;

	public printing_base(config conf) {
		super(conf);
		isotopic_mods = new ModifsList();
		this.fasta_type = null;
		this.set_max_progress(8);
	}

	protected void processing() throws Exception {
		// Ecrire le process et penser à relever des StopException
		logger.info("Printing to other format");

		msms_output output = new msms_output();
		isExpertMode = conf.get_export_expert_spreadsheet();
		if (conf.get_export_txt()) {
			this.set_view("Printing text tabulated file results");
			this.isCsv = true;
			this.spreadSheetReport();
		}

		if (conf.get_export_ods()) {
			this.set_view("Printing Open Document Spreadsheet file results");
			this.isCsv = false;
			this.spreadSheetReport();
		}

		if (conf.get_export_proticdbml()) {
			this.set_view("Printing PROTICdbML file results");

			FileOutputStream outputStream = new FileOutputStream(new File(this
					.getfileout().getAbsolutePath() + "_protic.xml"));

			ProticDbMlOutput protidDbMlOutput = new ProticDbMlOutput();
			protidDbMlOutput.open(outputStream);
			// massChroQmlOutput.write(outputStream);
			for (Identification ident : this.identifications) {
				protidDbMlOutput.write(ident);
			}
			protidDbMlOutput.close();

			// proticdbml protic = new proticdbml(new File(this.getfileout()
			// .getAbsolutePath() + "_protic.xml"));
			// for (Identification ident : this.identifications) {
			// protic.add_identification(ident);
			// }
		}

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_export_fasta()) {
			this.set_view("Printing fasta file protein result");
			HashSet<Protein> prot_deja_ecris = new HashSet<Protein>();
			output.open(this.getfileout().getAbsolutePath() + ".fasta");
			this.fasta_type = this.fasta_export_choice(output, prot_deja_ecris);
			for (Identification ident : this.identifications) {
				ident.print_result(this.fasta_type);
			}
			output.close();
		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_export_pepnovo()) {
			this.set_view("Printing PepNovo xml file results");
			output.open(this.getfileout().getAbsolutePath() + "_pepnovo.xml");
			output.write("<?xml version=\"1.0\"?>" + "\n"
					+ "<tandem_to_pepnovo>" + "\n");
			for (Identification ident : this.identifications) {
				ident.print_result(new print_pepnovo(output, ident
						.getMsRunSet()));
			}
			output.write("</tandem_to_pepnovo>" + "\n");
			output.close();
		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// Print de fdr...
		if ((conf.get_Export_fdr()) & (this.identifications.size() == 1)) {
			this.set_view("Printing FDR");
			output.open(this.getfileout().getAbsolutePath()
					+ "_fdr_peptide.txt");
			calculation_peptide_fdr_distribution calc = new calculation_peptide_fdr_distribution();
			calc.execute(this.identifications.get(0).get_protein_liste());
			calc.print_peptide_fdr_distribution(output);
			output.close();

			output.open(this.getfileout().getAbsolutePath()
					+ "_fdr_protein.txt");
			calculation_protein_fdr_distribution calc2 = new calculation_protein_fdr_distribution();
			calc2.execute(this.identifications.get(0).get_protein_liste());
			calc2.print_protein_fdr_distribution(output);
			output.close();

		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if ((conf.get_export_masschroq()) & (this.identifications.size() == 1)) {
			this.set_view("Printing MassChroQ xml file results");
			Identification ident = this.identifications.get(0);
			// modification isotopic
			ident.filtered_match(new filter_isotopic_modifs(this.isotopic_mods));

			MassChroQmlOutput massChroQmlOutput = new MassChroQmlOutput(
					this.isotopic_mods, ident);
			FileOutputStream outputStream = new FileOutputStream(new File(this
					.getfileout().getAbsolutePath() + ".masschroqML"));

			massChroQmlOutput.write(outputStream);
		}

		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

	}

	protected void spreadSheetReport() throws Exception {
		logger.info("spreadSheetReport begin");

		this.set_view("Printing protein file results");

		OdsDocWriter odsDoc = null;
		try {
			if (this.isCsv) {

			} else {
				odsDoc = new OdsDocWriter(this.getfileout().getAbsolutePath()
						+ ".ods");
			}

			logger.debug("Print new proteins");
			this.printProteinList(odsDoc);
			logger.debug("Print new peptide");
			this.printPeptideList(odsDoc);
			logger.debug("Print new spectra compar");
			this.printComparSpectra(odsDoc);
			logger.debug("Print new spectra specific");
			this.printComparSpecificSpectra(odsDoc);
			logger.debug("Print new compar pai");
			this.printComparPai(odsDoc);

			logger.debug("Print new spectra (expert)");
			this.printSpectraList(odsDoc);
			logger.debug("Print new peptide/prot (expert)");
			this.printPeptideProtList(odsDoc);
			// odsDoc.close();
		} catch (Exception e) {
			String message = "Error in spreadSheetReport :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message, e);
		} finally {
			if (odsDoc != null) {
				logger.debug("Closing odsDoc");
				odsDoc.close();
			}
		}
		this.add_current_progress();
	}

	protected void printComparPai(OdsDocWriter odsDoc) throws Exception {

	}

	protected void printComparSpecificSpectra(OdsDocWriter odsDoc)
			throws Exception {
	}

	protected void printComparSpectra(OdsDocWriter odsDoc) throws Exception {
	}

	private print_fasta fasta_export_choice(msms_output out,
			HashSet<Protein> prot_deja_ecris) {
		if (this.fasta_type == null) {
			this.fasta_type = new print_fasta();
			logger.debug("fasta export reinitialise");
		}
		this.fasta_type.setProt_deja_ecris(prot_deja_ecris);
		this.fasta_type.setOut(out);
		return (this.fasta_type);
	}

	public void setfileout(File out) {
		this.out = out;
	}

	public File getfileout() {
		return this.out;
	}

	public void setIsotopic_mods(ModifsList isotopic_mods) {
		this.isotopic_mods = isotopic_mods;
	}

	public ModifsList getIsotopic_mods() {
		return isotopic_mods;
	}

	public void setFasta_type(print_fasta fasta_type) {
		this.fasta_type = fasta_type;
		logger.debug(this.fasta_type.toString());
	}

	public print_fasta getFasta_type() {
		return fasta_type;
	}

	protected void printProteinList(OdsDocWriter odsDoc) throws Exception {
		logger.debug("printProteinList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing protein list results");
		if (this.isCsv) {
			Ods2Csv ods2csv = new Ods2Csv();
			ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
					+ "_proteins.txt");

			odsDoc = ods2csv.getOdsDocPipedOutput();
		} else {
		}

		OdsProteinOutput writer = new OdsProteinOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printProteinList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_proteins.txt");
		}
	}

	protected void printSpectraList(OdsDocWriter odsDoc) throws Exception {
		logger.debug("printSpectraList begin");
		if (this.isExpertMode) {

			// Verification pas arrete
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();

			this.set_view("Printing spectra list results");
			if (this.isCsv) {
				Ods2Csv ods2csv = new Ods2Csv();
				ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
						+ "_spectra.txt");

				odsDoc = ods2csv.getOdsDocPipedOutput();
			} else {
			}

			OdsSpectraOutput writer = new OdsSpectraOutput(odsDoc);
			// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
			for (Identification ident : this.identifications) {
				this.writeMsSampleName(ident, odsDoc);
				writer.write(ident);
			}
			logger.debug("printSpectraList end");

			if (this.isCsv) {
				odsDoc.close();
				logger.debug("csv file closed "
						+ this.getfileout().getAbsolutePath() + "_spectra.txt");
			}
		}
	}

	protected void printPeptideProtList(OdsDocWriter odsDoc) throws Exception {
		logger.debug("printPeptideProtList begin");
		if (this.isExpertMode) {

			// Verification pas arrete
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();

			this.set_view("Printing peptide/prot list results");
			if (this.isCsv) {
				Ods2Csv ods2csv = new Ods2Csv();
				ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
						+ "_peptides_pos.txt");

				odsDoc = ods2csv.getOdsDocPipedOutput();
			} else {
			}

			OdsPeptidePosOutput writer = new OdsPeptidePosOutput(odsDoc);
			// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
			for (Identification ident : this.identifications) {
				this.writeMsSampleName(ident, odsDoc);
				writer.write(ident);
			}
			logger.debug("printPeptideProtList end");

			if (this.isCsv) {
				odsDoc.close();
				logger.debug("csv file closed "
						+ this.getfileout().getAbsolutePath()
						+ "_peptides_pos.txt");
			}
		}
	}

	protected void writeMsSampleName(Identification ident, OdsDocWriter writer)
			throws Exception {

	}

	protected void printPeptideList(OdsDocWriter odsDoc) throws Exception {
		logger.debug("printPeptideList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing peptide list results");
		if (this.isCsv) {
			Ods2Csv ods2csv = new Ods2Csv();
			ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
					+ "_peptide_list.txt");

			odsDoc = ods2csv.getOdsDocPipedOutput();
		} else {
		}

		OdsPeptideOutput writer = new OdsPeptideOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPeptideList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_peptide_list.txt");
		}
	}
}
