package fr.inra.pappso.xtandempipeline.thread;

import fr.inra.pappso.csv2ods.Ods2Csv;
import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoComparSpectraOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoPeptideOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoProteinOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoSpectraOutput;

public class printing_phosphopeptide extends printing_base {

	public printing_phosphopeptide(config conf) {
		super(conf);
	}

	@Override
	protected void spreadSheetReport() throws Exception {
		logger.info("spreadSheetReport begin");

		this.set_view("Printing protein file results");

		OdsDocWriter odsDoc = null;
		try {
			if (this.isCsv) {

			} else {
				odsDoc = new OdsDocWriter(this.getfileout().getAbsolutePath()
						+ ".ods");
			}

			logger.debug("Print new proteins");
			this.printPhosphoProteinList(odsDoc);
			logger.debug("Print new peptides");
			this.printPhosphoPeptideList(odsDoc);
			logger.debug("Print new compar spectra");
			this.printPhosphoComparSpectraList(odsDoc);
			logger.debug("Print new spectra list");
			this.printPhosphoSpectraList(odsDoc);

			odsDoc.close();
		} catch (Exception e) {
			throw e;
		} finally {
			if (odsDoc != null) {
				odsDoc.close();
			}
		}
		this.add_current_progress();
	}

	private void printPhosphoSpectraList(OdsDocWriter odsDoc) throws Exception {
		logger.debug("printPhosphoSpectraList begin");

		if (this.isExpertMode) {
			// Verification pas arrete
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();

			this.set_view("Printing spectra results");
			if (this.isCsv) {
				Ods2Csv ods2csv = new Ods2Csv();
				ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
						+ "_spectra.txt");

				odsDoc = ods2csv.getOdsDocPipedOutput();
			} else {
			}

			OdsPhosphoSpectraOutput writer = new OdsPhosphoSpectraOutput(odsDoc);
			// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
			for (Identification ident : this.identifications) {
				this.writeMsSampleName(ident, odsDoc);
				writer.write(ident);
			}
			logger.debug("printPhosphoSpectraList end");

			if (this.isCsv) {
				odsDoc.close();
				logger.debug("csv file closed "
						+ this.getfileout().getAbsolutePath() + "_spectra.txt");
			}
		}
	}

	private void printPhosphoComparSpectraList(OdsDocWriter odsDoc)
			throws Exception {
		logger.debug("printPhosphoComparSpectraList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing compar spectra results");
		if (this.isCsv) {
			Ods2Csv ods2csv = new Ods2Csv();
			ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
					+ "_compar_spectra.txt");

			odsDoc = ods2csv.getOdsDocPipedOutput();
		} else {
		}

		OdsPhosphoComparSpectraOutput writer = new OdsPhosphoComparSpectraOutput(
				odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPhosphoComparSpectraList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath()
					+ "_compar_spectra.txt");
		}
	}

	private void printPhosphoPeptideList(OdsDocWriter odsDoc) throws Exception {
		logger.debug("printPhosphoPeptideList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing peptide list results");
		if (this.isCsv) {
			Ods2Csv ods2csv = new Ods2Csv();
			ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
					+ "_peptides.txt");

			odsDoc = ods2csv.getOdsDocPipedOutput();
		} else {
		}

		OdsPhosphoPeptideOutput writer = new OdsPhosphoPeptideOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPhosphoPeptideList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_peptides.txt");
		}
	}

	private void printPhosphoProteinList(OdsDocWriter odsDoc) throws Exception {
		logger.debug("printPhosphoProteinList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing protein list results");
		if (this.isCsv) {
			Ods2Csv ods2csv = new Ods2Csv();
			ods2csv.setSinkFilename(this.getfileout().getAbsolutePath()
					+ "_proteins.txt");

			odsDoc = ods2csv.getOdsDocPipedOutput();
		} else {
		}

		OdsPhosphoProteinOutput writer = new OdsPhosphoProteinOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPhosphoProteinList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_proteins.txt");
		}
	}

}
